(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    171983,       4636]
NotebookOptionsPosition[    163408,       4403]
NotebookOutlinePosition[    163785,       4420]
CellTagsIndexPosition[    163742,       4417]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Introduction", "Section",
 CellChangeTimes->{{3.502471550477194*^9, 3.502471552062395*^9}}],

Cell["\<\
This notebook implements and tests the Cayley-Dickson construction of complex \
numbers, quaternions, octonions, sedenions, and higher 2^n-ions.
0 - 1 - real
1 - 2 - complex
2 - 4 - quaternion
3 - 8 - octonion
4 - 16 - sedenion
etc.

The 2^n-ions are represented as lists that are 2^n long. Addition and scalar \
multiplication are component-by-component, and can use Mathematica's list \
operations.

Operators:
Conjugate:
CDConj[x_] -- theoretical construction
CDConjFast[x_] -- fast
Multiply:
CDMult[x_, y_] 
Norm (absolute square):
CDNorm[x_]  --  theoretical construction
CDNormFast[x_] -- fast
Reciprocal:
CDRecip[x_]
Real and imaginary parts:
CDReal[x_] -- commutes with all
CDImag[x_]
Zero and one; arg is total length:
CDZero[n_]
CDOne[n_]
Power:
CDPower[x_,p_] -- value and power

Generalized:
Additional parameter: list of multipliers for order = 1, 2, 3, ...
Multiply:
CDGMult[x_,y_,gl_]
Norm:
CDGNorm[x_,gl_]
Reciprocal:
CDGRecip[x_,gl_]
Power:
CDGPower[x_,p_,gl_] -- value and power

Multiplication tables of units: {1,0,0,...,0} ... {0,0,0,...,1}
Algebra power: np
List of mult-tab rules: {i1,i2} -> {i12,s}
where i1,i2,i12 are zero-based indices and s is the sign
Note: i12 is the bitwise xor of i1 and i2
CDUnitMult[np_]
List from power 0:
CDUnitMultList[np_]
Multiplication table for units as a n*n*n array (1st arg, 2nd arg, result):
CDUnitMultTab[np]
Use table tab to multiply coefficient vectors a and b:
MultWithTab[tab_,a_,b_]

Property tests:
Success produces zeros
CDSelfConj[x_]  -- self-conjugacy
CDComm[x_, y_] -- commutative property
CDAssoc[x_, y_, z_]  -- associative property
CDAlter[x_,y_] -- alternative property: associative over 2 values
CDNormMult[x_, y_] -- whether the norm of a product = product of norms
Can't think of a general way to prove power-associativity with Mathematica

Tests on algebras: real, complex, quaternion, octonion, sedenion
Definitions for real, complex, quaternion (Pauli matrices)
Self-conjugacy, commutativity, associativity, alternativity, norm \
multiplicativity
Either demonstrates in all cases or provides counterexamples

Real: self-conjugacy, commutativity, associativity, norm multiplicativity
Complex: commutativity, associativity, norm multiplicativity
Quaternions: associativity, norm multiplicativity
Octonions: alternativity, norm multiplicativity
Sedenions: none of these
Higher: like sedenions

Proof of power-associativity

Let x = P + Q*n
where n is imaginary: cjg(n) = - n, norm(n) = - n.n
with
x1 = P1 + Q1*n, etc.
Then, x12 = x1.x2 gives
P12 = P1*P2 - Q1*Q2*norm(n)
Q12 = P1*Q2 + P2*Q1
Let
R = P + Q*sqrt(-norm(n))
Then
R12 = R1*R2
For power p,
R(p) = R^p
Thus proving by construction.


Automorphism Symmetry Groups: continuous and discrete

Their elements are matrices that operate on the algebra-element vectors.
Real-real = 1, real-imag = imag=real = 0, imag-imag = the matrices to be \
discussed here.

From the norm, the automorphism groups are subgroups of O(2^np-1), the group \
of real orthogonal matrices.

One can find them with
CDAutoTest[mat_]

The ones with determinant 1 can be specified as exp(a.L) for Lie-algebra \
matrices L and coefficients a. Here, L is antisymmetric real matrices.

One can find them with
CDDiffAutoTest[mat_]

Real: identity group
Complex: Z2 -- identity and complex conjugate
Quaternions: SO(3) -- pure 3D rotations
Octonions and up: G2 -- the first exceptional Lie algebra

Continuous:

Quaternions: group SO(3) - (3) rep of algebra SO(3) ~ SU(2) ~ Sp(2)

Octonions and above: the generators are Ng copies of each of the G(2) \
generators, where Ng = (size)/8. These generators are in the (7) rep in \
algebra G(2). They are antisymmetric, making them a subset of the generators \
in the (7) rep of algebra SO(7).
Verified up to 32-ions.

Determinants of Hermitian matrices:
{{a,x*},{x,b}} = a*b - |x|^2
{{a,z*,y*},{z,b,x},{y,x*,c}} = a*b*c - (a*|x|^2+b*|y|^2+c*|z|^2) + 2Re(x.y.z)
Find automorphisms
for matrix sizes of order m = 2, 3, 4, ...
and CD constructions of order np = 0, 1, 2, ...

m = 2: SO(2^np+1,1) -- vector
np = 0: SL(m,R) -- symmetric 2-tensor (all values)
np = 1: SL(m,C) -- symmetric and antisymmetric 2-tensors (real, imag parts)
np = 2: SL(m,H) = SU*(2m) -- antisymmetric 2-tensor (off-diagonal values)
np = 3, m = 3: E6 analytic continuation (52+, 27-)

Discrete: Group of Units under multiplication
All members have all components zero but one, which is +- 1
0: Z(2) -- 2 -  real: {1,-1}
1: Z(4) -- 4 -  complex: {1,i,-1,-i}
2: Dih(8) -- 16 - quaternions

The index values combine by binary xor.
(Group of Units) -(Z(2)^n)-> Z(2) {1,-1}

Group of automorphisms of this group:
1: Z(2) -- 2 {i <-> -i}
2: Sym(4) -- 24

Aut(group of units) -(Aut(group of xor mults))-> Z(2)^n -- each bit's signs
Group of automorphisms of xor multiplies:
PSL(n,2) ~ PGL(n,2) ~ SL(n,2) ~ GL(n,2)
1: PSL(1, 2) = identity -- 1
2: PSL(2, 2) = Sym(3) = Dih(3) -- 6
3: PSL(3, 2) = PSL(2, 7) -- 168
4: PSL(4, 2) = Alt(8) -- 20160
Order: (2^n-1)(2^n-2)(2^n-4)...(2^n-2^(n-1))

Reference for much of this work: The Octonions by John Baez
http://math.ucr.edu/home/baez/octonions/\
\>", "Text",
 CellChangeTimes->{{3.5475298979871893`*^9, 3.54753008077129*^9}, {
   3.54753017372611*^9, 3.547530202283165*^9}, {3.547530415714658*^9, 
   3.547530418183764*^9}, {3.5475306585951023`*^9, 3.547530677528326*^9}, {
   3.5475308121248293`*^9, 3.5475308672182817`*^9}, {3.5475309914407883`*^9, 
   3.5475310172838173`*^9}, {3.547531201306691*^9, 3.547531206781658*^9}, {
   3.5475312458142443`*^9, 3.547531246550218*^9}, {3.547531547666258*^9, 
   3.547531593585977*^9}, {3.54753307967696*^9, 3.547533107410204*^9}, {
   3.5475331513381433`*^9, 3.547533193951112*^9}, {3.54753358372051*^9, 
   3.547533601878695*^9}, {3.547533706019862*^9, 3.5475337209074507`*^9}, 
   3.547562966941708*^9, {3.547563814754331*^9, 3.54756396242109*^9}, {
   3.547565563522661*^9, 3.547565651405645*^9}, {3.547565788842247*^9, 
   3.5475658506315403`*^9}, {3.547565883525317*^9, 3.547565907088315*^9}, {
   3.5475660948185263`*^9, 3.547566102954154*^9}, 3.5475664247144203`*^9, {
   3.547571977832374*^9, 3.547572016709584*^9}, {3.547581244329503*^9, 
   3.547581432749009*^9}, {3.547581466731731*^9, 3.547581499085511*^9}, {
   3.547586403760232*^9, 3.547586461961447*^9}, {3.547586654077599*^9, 
   3.5475866862210073`*^9}, {3.5475867610810003`*^9, 3.547586827739973*^9}, {
   3.547586876054433*^9, 3.547586900401857*^9}, {3.547587146186187*^9, 
   3.547587184847495*^9}, {3.5475876165733137`*^9, 3.547587627324089*^9}, {
   3.54758778378372*^9, 3.547587785117734*^9}, {3.547587831161227*^9, 
   3.547587880438744*^9}, {3.547587919473916*^9, 3.547587919815365*^9}, {
   3.54758807055093*^9, 3.5475881286903687`*^9}, {3.5475882086672897`*^9, 
   3.547588218266349*^9}, {3.547588282826692*^9, 3.5475882930672703`*^9}, {
   3.547588503084029*^9, 3.5475885496696997`*^9}, {3.54758872288941*^9, 
   3.547588747999861*^9}, {3.5475888457816133`*^9, 3.5475890825231543`*^9}, {
   3.547589132790942*^9, 3.547589133867897*^9}, {3.5491887464901257`*^9, 
   3.549188920186088*^9}, {3.549188989602557*^9, 3.549189003424859*^9}, {
   3.549191618300352*^9, 3.5491916640896263`*^9}, {3.549192336797336*^9, 
   3.5491923479953833`*^9}, {3.549192429429953*^9, 3.5491924299961576`*^9}, {
   3.549192760994297*^9, 3.549192771248521*^9}, {3.549192833151449*^9, 
   3.549192845968162*^9}, {3.549194097046404*^9, 3.5491941113165627`*^9}, 
   3.549195776387992*^9, {3.549195914704783*^9, 3.5491959783429337`*^9}, {
   3.5491963440851107`*^9, 3.549196352226632*^9}, {3.549197972284486*^9, 
   3.54919798268293*^9}, {3.549198015921986*^9, 3.5491980586046143`*^9}, {
   3.5491981411194487`*^9, 3.549198188140835*^9}, {3.54919836471353*^9, 
   3.549198371899024*^9}, {3.5492024775580893`*^9, 3.54920250392905*^9}, {
   3.591186257073196*^9, 3.591186292607472*^9}, {3.5911896521706667`*^9, 
   3.591189652625494*^9}, {3.591190120707699*^9, 3.591190171822854*^9}, {
   3.5914377409353523`*^9, 3.591437754456889*^9}, {3.591437864141741*^9, 
   3.591438172571191*^9}, {3.5914382066523857`*^9, 3.5914382083440104`*^9}, {
   3.591438959940733*^9, 3.5914389689817057`*^9}, {3.591452389449387*^9, 
   3.5914523954286127`*^9}, {3.5915138195757437`*^9, 3.591513823828166*^9}, {
   3.5915140161615343`*^9, 3.591514035622945*^9}, {3.591550146099023*^9, 
   3.591550177896305*^9}, {3.5915503804768667`*^9, 3.591550403195428*^9}, {
   3.59155057688834*^9, 3.5915505946688223`*^9}, {3.591550647680789*^9, 
   3.591550650437265*^9}, {3.59155103898866*^9, 3.591551069698413*^9}, {
   3.591551200959095*^9, 3.59155132235611*^9}, {3.591551366756895*^9, 
   3.591551381428985*^9}, {3.59155161153097*^9, 3.5915517010158873`*^9}, {
   3.591551732411251*^9, 3.591551742032333*^9}, {3.59155211273741*^9, 
   3.591552152020535*^9}, {3.591552185485339*^9, 3.5915522457497177`*^9}, {
   3.591552279393682*^9, 3.591552280614073*^9}, {3.591558175044422*^9, 
   3.591558175386601*^9}, {3.620251828479957*^9, 3.6202518382450333`*^9}, {
   3.620252916090509*^9, 3.620253051474167*^9}, {3.620253291589986*^9, 
   3.620253362719277*^9}, {3.620253555801218*^9, 3.6202536860654507`*^9}, {
   3.6202546854060163`*^9, 3.620254702437229*^9}, {3.620256000780098*^9, 
   3.6202560028359423`*^9}, {3.6202560704717627`*^9, 3.620256131918494*^9}, {
   3.620256165459709*^9, 3.620256174381462*^9}, {3.6202563127111797`*^9, 
   3.620256402616102*^9}, 3.620256475542062*^9, {3.620256781316505*^9, 
   3.6202567894364967`*^9}, {3.6202585934734917`*^9, 3.620258598249381*^9}, {
   3.620261096248321*^9, 3.620261205902857*^9}, {3.6202612388151903`*^9, 
   3.620261274810505*^9}, {3.620298225387084*^9, 3.62029822572787*^9}, {
   3.621475675749908*^9, 3.6214757390627604`*^9}, {3.621477896222292*^9, 
   3.621477911942204*^9}, {3.6214779632088842`*^9, 3.621477977670348*^9}, {
   3.6215032592708263`*^9, 3.6215032868500566`*^9}, {3.6215034677830143`*^9, 
   3.621503505672278*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Operators", "Section",
 CellChangeTimes->{{3.547529879553905*^9, 3.547529880911971*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Conjugate", ":"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.547529777500545*^9, 3.547529782304175*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDConj", "[", "x_List", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"n", ",", "nh", ",", "xl", ",", "xh"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"n", " ", "=", " ", 
      RowBox[{"Length", "[", "x", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"n", " ", "\[Equal]", " ", "1"}], ",", 
       RowBox[{"Return", "[", "x", "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"nh", " ", "=", " ", 
      RowBox[{"n", "/", "2"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"!", 
        RowBox[{"IntegerQ", "[", "nh", "]"}]}], ",", 
       RowBox[{"Return", "[", "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"xl", " ", "=", " ", 
      RowBox[{"Take", "[", 
       RowBox[{"x", ",", "nh"}], "]"}]}], ";", " ", 
     RowBox[{"xh", " ", "=", " ", 
      RowBox[{"Take", "[", 
       RowBox[{"x", ",", 
        RowBox[{"-", "nh"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Join", "[", 
      RowBox[{
       RowBox[{"CDConj", "[", "xl", "]"}], ",", 
       RowBox[{"-", "xh"}]}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.5475297834490757`*^9, 3.5475298650515738`*^9}, {
   3.547530091278179*^9, 3.547530162160431*^9}, {3.547530210884117*^9, 
   3.54753021351291*^9}, 3.591438854561759*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDConjFast", "[", "x_List", "]"}], " ", ":=", " ", 
  RowBox[{"Prepend", "[", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"Rest", "[", "x", "]"}]}], ",", 
    RowBox[{"First", "[", "x", "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5475302174414988`*^9, 3.5475302290934553`*^9}, 
   3.591438850011634*^9}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Multiply", ":"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.54753060332445*^9, 3.5475306071368113`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDMult", "[", 
   RowBox[{"x_List", ",", "y_List"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"n", ",", "nh", ",", "xl", ",", "xh", ",", "yl", ",", "yh"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"n", " ", "=", " ", 
      RowBox[{"Length", "[", "x", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "y", "]"}], " ", "\[NotEqual]", " ", "n"}], 
       ",", 
       RowBox[{"Return", "[", "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"n", " ", "\[Equal]", " ", "1"}], ",", 
       RowBox[{"Return", "[", 
        RowBox[{"x", "*", "y"}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"nh", " ", "=", " ", 
      RowBox[{"n", "/", "2"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"!", 
        RowBox[{"IntegerQ", "[", "nh", "]"}]}], ",", 
       RowBox[{"Return", "[", "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"xl", ",", "xh"}], "}"}], " ", "=", " ", 
      RowBox[{"Partition", "[", 
       RowBox[{"x", ",", "nh"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"yl", ",", "yh"}], "}"}], " ", "=", " ", 
      RowBox[{"Partition", "[", 
       RowBox[{"y", ",", "nh"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Join", "[", 
      RowBox[{
       RowBox[{"Expand", "[", 
        RowBox[{
         RowBox[{"CDMult", "[", 
          RowBox[{"xl", ",", "yl"}], "]"}], " ", "-", " ", 
         RowBox[{"CDMult", "[", 
          RowBox[{
           RowBox[{"CDConj", "[", "yh", "]"}], ",", "xh"}], "]"}]}], "]"}], 
       ",", " ", 
       RowBox[{"Expand", "[", 
        RowBox[{
         RowBox[{"CDMult", "[", 
          RowBox[{"yh", ",", "xl"}], "]"}], " ", "+", " ", 
         RowBox[{"CDMult", "[", 
          RowBox[{"xh", ",", " ", 
           RowBox[{"CDConj", "[", "yl", "]"}]}], "]"}]}], "]"}]}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.547530263508314*^9, 3.54753040212663*^9}, {
   3.547532735047576*^9, 3.547532748601413*^9}, {3.5475338850077753`*^9, 
   3.547533885363603*^9}, {3.547566181834628*^9, 3.5475661908689613`*^9}, {
   3.591438846026619*^9, 3.591438848409452*^9}, 3.591582718919468*^9}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Absolute", " ", "square", " ", 
   RowBox[{"(", 
    RowBox[{"square", " ", "of", " ", "norm"}], ")"}]}], " ", "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDNorm", "[", "x_List", "]"}], " ", ":=", " ", 
  RowBox[{"First", "[", 
   RowBox[{"CDMult", "[", 
    RowBox[{"x", ",", 
     RowBox[{"CDConj", "[", "x", "]"}]}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.547530611119779*^9, 3.5475306285055532`*^9}, 
   3.5475307893560553`*^9, {3.547532405583744*^9, 3.547532408252434*^9}, 
   3.5914388606231327`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDNormFast", "[", "x_List", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"Total", "[", 
    RowBox[{"x", "^", "2"}], "]"}], " ", "//", " ", "Expand"}]}]], "Input",
 CellChangeTimes->{{3.5475306308556747`*^9, 3.5475306477375307`*^9}, 
   3.5475307921907883`*^9, {3.547566198896209*^9, 3.54756620030101*^9}, 
   3.5914388623096333`*^9}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Finds", " ", 
   RowBox[{"reciprocal", ":", " ", 
    RowBox[{"useful", " ", "for", " ", "division"}]}]}], " ", "*)"}]], "Input",\

 CellChangeTimes->{{3.547566119127819*^9, 3.547566128883996*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDRecip", "[", "x_List", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"CDConj", "[", "x", "]"}], "/", 
    RowBox[{"CDNorm", "[", "x", "]"}]}], " ", "//", " ", 
   "Expand"}]}]], "Input",
 CellChangeTimes->{{3.547566132405653*^9, 3.547566145652956*^9}, {
   3.547566206008223*^9, 3.5475662074692993`*^9}, 3.5914388643784933`*^9}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Real", " ", "and", " ", "imaginary", " ", "parts"}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.5475308777366333`*^9, 3.5475308842841377`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDReal", "[", "x_List", "]"}], " ", ":=", " ", 
  RowBox[{"Prepend", "[", 
   RowBox[{
    RowBox[{"Table", "[", 
     RowBox[{"0", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Length", "[", "x", "]"}], "-", "1"}], "}"}]}], "]"}], ",", 
    RowBox[{"First", "[", "x", "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5475308945794697`*^9, 3.547530902084239*^9}, {
   3.547531083673149*^9, 3.547531095383503*^9}, {3.547531404690699*^9, 
   3.5475314194356947`*^9}, 3.5914388657220182`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDImag", "[", "x_List", "]"}], " ", ":=", " ", 
  RowBox[{"Prepend", "[", 
   RowBox[{
    RowBox[{"Rest", "[", "x", "]"}], ",", "0"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.547530903578752*^9, 3.5475309169009132`*^9}, 
   3.591438867522039*^9}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Zero", " ", "and", " ", "one", " ", "from", " ", "total", " ", "length"}], 
  " ", "*)"}]], "Input",
 CellChangeTimes->{{3.5475309196867943`*^9, 3.5475309327969847`*^9}, {
  3.5475315752298307`*^9, 3.547531579067243*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDZero", "[", "n_Integer", "]"}], " ", ":=", " ", 
  RowBox[{"ConstantArray", "[", 
   RowBox[{"0", ",", "n"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.547530933890213*^9, 3.547530949496913*^9}, {
  3.547531601838873*^9, 3.547531602163714*^9}, {3.591438869522524*^9, 
  3.591438882438591*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDOne", "[", "n_Integer", "]"}], " ", ":=", " ", 
  RowBox[{"Prepend", "[", 
   RowBox[{
    RowBox[{"ConstantArray", "[", 
     RowBox[{"0", ",", 
      RowBox[{"n", "-", "1"}]}], "]"}], ",", "1"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.547531607236618*^9, 3.547531616284213*^9}, {
  3.591438889353479*^9, 3.591438895034487*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", "Power", " ", "*)"}]], "Input",
 CellChangeTimes->{{3.5475312713305693`*^9, 3.547531275231196*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDPower", "[", 
   RowBox[{"x_List", ",", "p_Integer"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "one", ",", "xr", ",", "xi", ",", "xisq", ",", "k", ",", "pwc0", ",", 
      "pwc1", ",", "oldpwc0", ",", " ", "oldpwc1"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"one", " ", "=", " ", 
      RowBox[{"CDOne", "[", 
       RowBox[{"Length", "[", "x", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Negative", " ", "power"}], " ", "*)"}], "\[IndentingNewLine]", 
     
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"p", " ", "<", " ", "0"}], ",", " ", 
       RowBox[{"Return", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"CDNorm", "[", "x", "]"}], "^", "p"}], "*", 
         RowBox[{"CDPower", "[", 
          RowBox[{
           RowBox[{"CDConj", "[", "x", "]"}], ",", 
           RowBox[{"-", "p"}]}], "]"}]}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{"Nonnegative", " ", "power"}], " ", "-", " ", 
       RowBox[{"can", " ", "be", " ", "zero"}]}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"xr", " ", "=", " ", 
      RowBox[{"First", "[", "x", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"xi", " ", "=", " ", 
      RowBox[{"CDImag", "[", "x", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"xisq", " ", "=", " ", 
      RowBox[{"CDNorm", "[", "xi", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{"One", " ", "could", " ", "calculate", " ", 
        RowBox[{"directly", ":", "\[IndentingNewLine]", "pwc0"}]}], " ", "=", 
       " ", 
       RowBox[{
        RowBox[{"sum", " ", "of", " ", "binomial", 
         RowBox[{"(", 
          RowBox[{"n", ",", 
           RowBox[{"2", "k"}]}], ")"}], "*", 
         RowBox[{"xr", "^", 
          RowBox[{"(", 
           RowBox[{"n", "-", 
            RowBox[{"2", "k"}]}], ")"}]}], "*", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"-", "xi"}], ")"}], "^", "k"}], "\[IndentingNewLine]", 
         "pwc1"}], " ", "=", " ", 
        RowBox[{"sum", " ", "of", " ", "binomial", 
         RowBox[{"(", 
          RowBox[{"n", ",", 
           RowBox[{
            RowBox[{"2", "k"}], "+", "1"}]}], ")"}], "*", 
         RowBox[{"xr", "^", 
          RowBox[{"(", 
           RowBox[{"n", "-", 
            RowBox[{"2", "k"}], "-", "1"}], ")"}]}], "*", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"-", "xi"}], ")"}], "^", "k"}]}]}]}], 
      "\[IndentingNewLine]", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"pwc0", " ", "=", " ", "1"}], ";", " ", 
     RowBox[{"pwc1", " ", "=", " ", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"oldpwc0", " ", "=", " ", "pwc0"}], ";", " ", 
        RowBox[{"oldpwc1", " ", "=", " ", "pwc1"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"pwc0", " ", "=", " ", 
         RowBox[{"Expand", "[", 
          RowBox[{
           RowBox[{"xr", "*", "oldpwc0"}], " ", "-", " ", 
           RowBox[{"xisq", "*", "oldpwc1"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"pwc1", " ", "=", " ", 
         RowBox[{"Expand", "[", 
          RowBox[{
           RowBox[{"xr", "*", "oldpwc1"}], " ", "+", " ", "oldpwc0"}], 
          "]"}]}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"k", ",", "p"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Expand", "[", 
      RowBox[{
       RowBox[{"pwc0", "*", "one"}], " ", "+", " ", 
       RowBox[{"pwc1", "*", "xi"}]}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.547531277837779*^9, 3.5475312833450937`*^9}, {
   3.547531324752829*^9, 3.547531355681747*^9}, {3.5475313882981253`*^9, 
   3.547531396074831*^9}, {3.547531447270938*^9, 3.547531482246581*^9}, {
   3.547531524718074*^9, 3.547531532966441*^9}, {3.547531630122456*^9, 
   3.547531646295141*^9}, {3.547531677066029*^9, 3.547531694368972*^9}, {
   3.547531754338735*^9, 3.5475318379723377`*^9}, {3.547531889496696*^9, 
   3.54753198788645*^9}, {3.547532044583439*^9, 3.547532184635724*^9}, {
   3.547532289742505*^9, 3.547532294644754*^9}, {3.5475323382070208`*^9, 
   3.547532343069483*^9}, {3.547532389975045*^9, 3.547532422286201*^9}, {
   3.547532599020563*^9, 3.547532679999967*^9}, {3.5914387885529757`*^9, 
   3.591438790306508*^9}, 3.591438830987104*^9}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Multiplication", " ", "table", " ", "of", " ", "units"}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.6202516464298067`*^9, 3.620251653295796*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Input", " ", "and", " ", 
    RowBox[{"output", ":", " ", 
     RowBox[{"power", " ", "of", " ", "2"}]}]}], ",", " ", 
   RowBox[{
    RowBox[{"list", " ", "of", " ", "rules", " ", "in", " ", "form", " ", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"index", " ", "1"}], ",", " ", 
       RowBox[{"index", " ", "2"}]}], "}"}]}], " ", "\[Rule]", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"result", " ", "index"}], ",", " ", "sign"}], "}"}], " ", 
      "where", " ", "the", " ", "indexing", " ", "is", " ", "zero"}], "-", 
     "based"}]}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.620251685109685*^9, 3.620251722433311*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDUnitMultStep", "[", "data_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "np", ",", "rls", ",", "newnp", " ", ",", "newrls", ",", "n", ",", "ih1",
       ",", "il1", ",", "ih2", ",", "il2", ",", "i1", ",", "i2", ",", "i12", 
      ",", "s"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"np", ",", "rls"}], "}"}], " ", "=", " ", "data"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"newnp", " ", "=", " ", 
      RowBox[{"np", "+", "1"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"n", " ", "=", " ", 
      RowBox[{"2", "^", "np"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"newrls", " ", "=", " ", 
      RowBox[{"Sort", "[", 
       RowBox[{"Flatten", "[", 
        RowBox[{"Table", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"i1", " ", "=", " ", 
            RowBox[{
             RowBox[{"n", "*", "ih1"}], "+", "il1"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"i2", " ", "=", " ", 
            RowBox[{
             RowBox[{"n", "*", "ih2"}], "+", "il2"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"i12", ",", "s"}], "}"}], " ", "=", " ", 
            RowBox[{
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"ih2", ">", "0"}], ",", 
               RowBox[{"{", 
                RowBox[{"il2", ",", "il1"}], "}"}], " ", ",", " ", 
               RowBox[{"{", 
                RowBox[{"il1", ",", "il2"}], "}"}]}], "]"}], " ", "/.", " ", 
             "rls"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"ih1", "\[NotEqual]", "ih2"}], ",", 
             RowBox[{"i12", "+=", "n"}]}], "]"}], ";", "\[IndentingNewLine]", 
           
           RowBox[{"s", " ", "*=", " ", 
            RowBox[{
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"ih1", ">", "0"}], ",", "  ", 
               RowBox[{"If", "[", 
                RowBox[{
                 RowBox[{"il2", "\[Equal]", "0"}], ",", " ", "1", ",", " ", 
                 RowBox[{"-", "1"}]}], "]"}], ",", " ", "1"}], "]"}], " ", 
             "*", "\[IndentingNewLine]", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"ih1", " ", ">", " ", "0"}], " ", "&&", " ", 
                RowBox[{"ih2", " ", ">", " ", "0"}]}], ",", " ", 
               RowBox[{"-", "1"}], ",", " ", "1"}], "]"}]}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"i1", ",", "i2"}], "}"}], "\[Rule]", 
            RowBox[{"{", 
             RowBox[{"i12", ",", "s"}], "}"}]}]}], ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"ih1", ",", "0", ",", "1"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"ih2", ",", "0", ",", "1"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"il1", ",", "0", ",", 
            RowBox[{"n", "-", "1"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"il2", ",", "0", ",", 
            RowBox[{"n", "-", "1"}]}], "}"}]}], "]"}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"newnp", ",", "newrls"}], "}"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.620221113438829*^9, 3.620221136746541*^9}, {
  3.620221175619218*^9, 3.6202212241587877`*^9}, {3.6202212764781218`*^9, 
  3.620221354490531*^9}, {3.620221396216325*^9, 3.620221444890142*^9}, {
  3.6202215083234*^9, 3.620221679533091*^9}, {3.620221786424366*^9, 
  3.620221894921845*^9}, {3.620221980039419*^9, 3.620221984644515*^9}, {
  3.620222029550762*^9, 3.620222151767757*^9}, {3.620254150152104*^9, 
  3.6202541648752604`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDUnitMultList", "[", "np_", "]"}], " ", ":=", " ", 
  RowBox[{"Last", " ", "/@", " ", 
   RowBox[{"NestList", "[", 
    RowBox[{"CDUnitMultStep", ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}], "\[Rule]", 
         RowBox[{"{", 
          RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], "}"}], ",", "np"}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.620222176288931*^9, 3.6202222158705482`*^9}, {
  3.620251740722149*^9, 3.620251744860024*^9}, {3.6202534128043423`*^9, 
  3.6202534147998037`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDUnitMult", "[", "np_", "]"}], " ", ":=", " ", 
  RowBox[{"Last", " ", "@", " ", 
   RowBox[{"Nest", "[", 
    RowBox[{"CDUnitMultStep", ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}], "\[Rule]", 
         RowBox[{"{", 
          RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], "}"}], ",", "np"}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.620222176288931*^9, 3.6202222158705482`*^9}, {
  3.620253418414176*^9, 3.620253419779027*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDUnitMultTab", "[", "np_", "]"}], " ", ":=", " ", 
  RowBox[{"SparseArray", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"#", "[", 
          RowBox[{"[", 
           RowBox[{"1", ",", "1"}], "]"}], "]"}], "+", "1"}], ",", 
        RowBox[{
         RowBox[{"#", "[", 
          RowBox[{"[", 
           RowBox[{"1", ",", "2"}], "]"}], "]"}], "+", "1"}], ",", 
        RowBox[{
         RowBox[{"#", "[", 
          RowBox[{"[", 
           RowBox[{"2", ",", "1"}], "]"}], "]"}], "+", "1"}]}], "}"}], 
      "\[Rule]", 
      RowBox[{"#", "[", 
       RowBox[{"[", 
        RowBox[{"2", ",", "2"}], "]"}], "]"}]}], "&"}], " ", "/@", "  ", 
    RowBox[{"CDUnitMult", "[", "np", "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.620224421606105*^9, 3.6202244738723717`*^9}, {
  3.6202534378830748`*^9, 3.6202534579877853`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDUnitMultTabVerify", "[", "np_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "n", ",", "vecs", ",", "v1", ",", "v2", ",", "tab0", ",", "tab1"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"n", " ", "=", " ", 
      RowBox[{"2", "^", "np"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"vecs", " ", "=", " ", 
      RowBox[{"IdentityMatrix", "[", "n", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"tab0", " ", "=", " ", 
      RowBox[{"SparseArray", "[", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"SparseArray", "[", 
          RowBox[{
           RowBox[{"CDMult", "[", 
            RowBox[{"v1", ",", "v2"}], "]"}], ",", 
           RowBox[{"{", "n", "}"}]}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"v1", ",", "vecs"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"v2", ",", "vecs"}], "}"}]}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"tab1", " ", "=", " ", 
      RowBox[{"CDUnitMultTab", "[", "np", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"SparseArray", "[", 
      RowBox[{"tab1", "-", "tab0"}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.620224502654832*^9, 3.6202246027543707`*^9}, {
  3.620226612426771*^9, 3.620226626496601*^9}, {3.620226680117797*^9, 
  3.620226687056196*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDUnitMultSign", "[", "np_", "]"}], " ", ":=", " ", 
  RowBox[{"SparseArray", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"#", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "+", "1"}], ")"}], "\[Rule]", 
      RowBox[{"#", "[", 
       RowBox[{"[", 
        RowBox[{"2", ",", "2"}], "]"}], "]"}]}], "&"}], " ", "/@", " ", 
    RowBox[{"CDUnitMult", "[", "np", "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.620695622269059*^9, 3.620695671484028*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"MakeBinDigitsStep", "[", "bdg_", "]"}], " ", ":=", " ", 
  RowBox[{"Flatten", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Append", "[", 
         RowBox[{"#", ",", "0"}], "]"}], ",", 
        RowBox[{"Append", "[", 
         RowBox[{"#", ",", "1"}], "]"}]}], "}"}], "&"}], " ", "/@", " ", 
     "bdg"}], ",", "1"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6202543903058023`*^9, 3.620254449991538*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"MakeBinDigits", "[", "np_", "]"}], " ", ":=", " ", 
  RowBox[{"Nest", "[", 
   RowBox[{"MakeBinDigitsStep", ",", 
    RowBox[{"{", 
     RowBox[{"{", "}"}], "}"}], ",", "np"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.620254360067168*^9, 3.620254385942443*^9}, {
  3.620254453591316*^9, 3.620254462554508*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDUnitMultIndexVerify", "[", "np_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "n", ",", "rls", ",", "i1", ",", "i2", ",", "pwrtwos", ",", "bdgs", ",", 
      "b1", ",", "b2", ",", "indtab0", ",", "indtab1"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"n", " ", "=", " ", 
      RowBox[{"2", "^", "np"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"rls", " ", "=", " ", 
      RowBox[{"CDUnitMult", "[", "np", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"indtab0", " ", "=", " ", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"First", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"i1", ",", "i2"}], "}"}], " ", "/.", " ", "rls"}], "]"}], 
        ",", 
        RowBox[{"{", 
         RowBox[{"i1", ",", "0", ",", 
          RowBox[{"n", "-", "1"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"i2", ",", "0", ",", 
          RowBox[{"n", "-", "1"}]}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"pwrtwos", " ", "=", " ", 
      RowBox[{"Reverse", "[", 
       RowBox[{"NestList", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"2", "*", "#"}], "&"}], ",", "1", ",", 
         RowBox[{"np", "-", "1"}]}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"bdgs", " ", "=", " ", 
      RowBox[{"MakeBinDigits", "[", "np", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"indtab1", " ", "=", " ", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"pwrtwos", ".", 
         RowBox[{"Mod", "[", 
          RowBox[{
           RowBox[{"b1", "+", "b2"}], ",", "2"}], "]"}]}], ",", 
        RowBox[{"{", 
         RowBox[{"b1", ",", "bdgs"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"b2", ",", "bdgs"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"SparseArray", "[", 
      RowBox[{"indtab1", "-", "indtab0"}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.6202541161036873`*^9, 3.6202541396279182`*^9}, {
  3.620254193732234*^9, 3.620254356017209*^9}, {3.620254514981143*^9, 
  3.620254628147592*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"MultWithTab", "[", 
   RowBox[{"tab_", ",", "a_", ",", "b_"}], "]"}], " ", ":=", " ", 
  RowBox[{"a", ".", 
   RowBox[{"(", 
    RowBox[{"b", ".", 
     RowBox[{"Transpose", "[", "tab", "]"}]}], ")"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{"(*", " ", "*)"}]], "Input",
 CellChangeTimes->{{3.6202532390230618`*^9, 3.620253239890135*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Generalized", " ", 
   RowBox[{"multiply", ":"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.54753060332445*^9, 3.5475306071368113`*^9}, {
  3.591438422145215*^9, 3.591438439730089*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDGMult", "[", 
   RowBox[{"x_List", ",", "y_List", ",", "gl_List"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "n", ",", "nh", ",", "xl", ",", "xh", ",", "yl", ",", "yh", ",", "gl0", 
      ",", "gl1"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"n", " ", "=", " ", 
      RowBox[{"Length", "[", "x", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "y", "]"}], " ", "\[NotEqual]", " ", "n"}], 
       ",", 
       RowBox[{"Return", "[", "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"n", " ", "\[Equal]", " ", "1"}], ",", 
       RowBox[{"Return", "[", 
        RowBox[{"x", "*", "y"}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"nh", " ", "=", " ", 
      RowBox[{"n", "/", "2"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"!", 
        RowBox[{"IntegerQ", "[", "nh", "]"}]}], ",", 
       RowBox[{"Return", "[", "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"xl", ",", "xh"}], "}"}], " ", "=", " ", 
      RowBox[{"Partition", "[", 
       RowBox[{"x", ",", "nh"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"yl", ",", "yh"}], "}"}], " ", "=", " ", 
      RowBox[{"Partition", "[", 
       RowBox[{"y", ",", "nh"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"gl0", " ", "=", " ", 
      RowBox[{"Most", "[", "gl", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"gl1", " ", "=", " ", 
      RowBox[{"Last", "[", "gl", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Join", "[", 
      RowBox[{
       RowBox[{"Expand", "[", 
        RowBox[{
         RowBox[{"CDGMult", "[", 
          RowBox[{"xl", ",", "yl", ",", "gl0"}], "]"}], " ", "-", " ", 
         RowBox[{"gl1", "*", 
          RowBox[{"CDGMult", "[", 
           RowBox[{
            RowBox[{"CDConj", "[", "yh", "]"}], ",", "xh", ",", "gl0"}], 
           "]"}]}]}], "]"}], ",", " ", 
       RowBox[{"Expand", "[", 
        RowBox[{
         RowBox[{"CDGMult", "[", 
          RowBox[{"yh", ",", "xl", ",", "gl0"}], "]"}], " ", "+", " ", 
         RowBox[{"CDGMult", "[", 
          RowBox[{"xh", ",", " ", 
           RowBox[{"CDConj", "[", "yl", "]"}], ",", "gl0"}], "]"}]}], "]"}]}],
       "]"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.547530263508314*^9, 3.54753040212663*^9}, {
  3.547532735047576*^9, 3.547532748601413*^9}, {3.5475338850077753`*^9, 
  3.547533885363603*^9}, {3.547566181834628*^9, 3.5475661908689613`*^9}, {
  3.591438271284666*^9, 3.591438279333688*^9}, {3.591438329322501*^9, 
  3.5914383800490627`*^9}, {3.591438834563883*^9, 3.591438838044964*^9}, {
  3.591451943909294*^9, 3.591451947356182*^9}, {3.591582666103105*^9, 
  3.5915826900067253`*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Generalized", " ", "absolute", " ", "square", " ", 
   RowBox[{"(", 
    RowBox[{"square", " ", "of", " ", "norm"}], ")"}]}], " ", "*)"}]], "Input",\

 CellChangeTimes->{{3.591438433868638*^9, 3.591438436305888*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDGNorm", "[", 
   RowBox[{"x_List", ",", "gl_List"}], "]"}], " ", ":=", " ", 
  RowBox[{"First", "[", 
   RowBox[{"CDGMult", "[", 
    RowBox[{"x", ",", 
     RowBox[{"CDConj", "[", "x", "]"}], ",", "gl"}], "]"}], "]"}]}]], "Input",\

 CellChangeTimes->{{3.547530611119779*^9, 3.5475306285055532`*^9}, 
   3.5475307893560553`*^9, {3.547532405583744*^9, 3.547532408252434*^9}, {
   3.591438403292582*^9, 3.591438415308536*^9}, {3.5914388189749823`*^9, 
   3.591438820882365*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Generalized", " ", "-", " ", 
   RowBox[{"finds", " ", 
    RowBox[{"reciprocal", ":", " ", 
     RowBox[{"useful", " ", "for", " ", "division"}]}]}]}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.547566119127819*^9, 3.547566128883996*^9}, {
  3.591438453442328*^9, 3.591438459943137*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDGRecip", "[", 
   RowBox[{"x_List", ",", "gl_List"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"CDConj", "[", "x", "]"}], "/", 
    RowBox[{"CDNorm", "[", 
     RowBox[{"x", ",", "gl"}], "]"}]}], " ", "//", " ", "Expand"}]}]], "Input",\

 CellChangeTimes->{{3.547566132405653*^9, 3.547566145652956*^9}, {
  3.547566206008223*^9, 3.5475662074692993`*^9}, {3.591438463665228*^9, 
  3.5914384718761597`*^9}, {3.5914388228348846`*^9, 3.5914388245479193`*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Generalized", " ", "power"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.5475312713305693`*^9, 3.547531275231196*^9}, {
  3.591438490836197*^9, 3.591438492867825*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDGPower", "[", 
   RowBox[{"x_List", ",", "p_Integer", ",", "gl_List"}], "]"}], " ", ":=", 
  " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "one", ",", "xr", ",", "xi", ",", "xisq", ",", "k", ",", "pwc0", ",", 
      "pwc1", ",", "oldpwc0", ",", " ", "oldpwc1"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"one", " ", "=", " ", 
      RowBox[{"CDOne", "[", 
       RowBox[{"Length", "[", "x", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Negative", " ", "power"}], " ", "*)"}], "\[IndentingNewLine]", 
     
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"p", " ", "<", " ", "0"}], ",", " ", 
       RowBox[{"Return", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"CDGNorm", "[", 
           RowBox[{"x", ",", "gl"}], "]"}], "^", "p"}], "*", 
         RowBox[{"CDGPower", "[", 
          RowBox[{
           RowBox[{"CDConj", "[", "x", "]"}], ",", 
           RowBox[{"-", "p"}], ",", "gl"}], "]"}]}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{"Nonnegative", " ", "power"}], " ", "-", " ", 
       RowBox[{"can", " ", "be", " ", "zero"}]}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"xr", " ", "=", " ", 
      RowBox[{"First", "[", "x", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"xi", " ", "=", " ", 
      RowBox[{"CDImag", "[", "x", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"xisq", " ", "=", " ", 
      RowBox[{"CDGNorm", "[", 
       RowBox[{"xi", ",", "gl"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{"One", " ", "could", " ", "calculate", " ", 
        RowBox[{"directly", ":", "\[IndentingNewLine]", "pwc0"}]}], " ", "=", 
       " ", 
       RowBox[{
        RowBox[{"sum", " ", "of", " ", "binomial", 
         RowBox[{"(", 
          RowBox[{"n", ",", 
           RowBox[{"2", "k"}]}], ")"}], "*", 
         RowBox[{"xr", "^", 
          RowBox[{"(", 
           RowBox[{"n", "-", 
            RowBox[{"2", "k"}]}], ")"}]}], "*", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"-", "xi"}], ")"}], "^", "k"}], "\[IndentingNewLine]", 
         "pwc1"}], " ", "=", " ", 
        RowBox[{"sum", " ", "of", " ", "binomial", 
         RowBox[{"(", 
          RowBox[{"n", ",", 
           RowBox[{
            RowBox[{"2", "k"}], "+", "1"}]}], ")"}], "*", 
         RowBox[{"xr", "^", 
          RowBox[{"(", 
           RowBox[{"n", "-", 
            RowBox[{"2", "k"}], "-", "1"}], ")"}]}], "*", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"-", "xi"}], ")"}], "^", "k"}]}]}]}], 
      "\[IndentingNewLine]", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"pwc0", " ", "=", " ", "1"}], ";", " ", 
     RowBox[{"pwc1", " ", "=", " ", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"oldpwc0", " ", "=", " ", "pwc0"}], ";", " ", 
        RowBox[{"oldpwc1", " ", "=", " ", "pwc1"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"pwc0", " ", "=", " ", 
         RowBox[{"Expand", "[", 
          RowBox[{
           RowBox[{"xr", "*", "oldpwc0"}], " ", "-", " ", 
           RowBox[{"xisq", "*", "oldpwc1"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"pwc1", " ", "=", " ", 
         RowBox[{"Expand", "[", 
          RowBox[{
           RowBox[{"xr", "*", "oldpwc1"}], " ", "+", " ", "oldpwc0"}], 
          "]"}]}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"k", ",", "p"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Expand", "[", 
      RowBox[{
       RowBox[{"pwc0", "*", "one"}], " ", "+", " ", 
       RowBox[{"pwc1", "*", "xi"}]}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.547531277837779*^9, 3.5475312833450937`*^9}, {
  3.547531324752829*^9, 3.547531355681747*^9}, {3.5475313882981253`*^9, 
  3.547531396074831*^9}, {3.547531447270938*^9, 3.547531482246581*^9}, {
  3.547531524718074*^9, 3.547531532966441*^9}, {3.547531630122456*^9, 
  3.547531646295141*^9}, {3.547531677066029*^9, 3.547531694368972*^9}, {
  3.547531754338735*^9, 3.5475318379723377`*^9}, {3.547531889496696*^9, 
  3.54753198788645*^9}, {3.547532044583439*^9, 3.547532184635724*^9}, {
  3.547532289742505*^9, 3.547532294644754*^9}, {3.5475323382070208`*^9, 
  3.547532343069483*^9}, {3.547532389975045*^9, 3.547532422286201*^9}, {
  3.547532599020563*^9, 3.547532679999967*^9}, {3.591438483029292*^9, 
  3.591438484754572*^9}, {3.591438520212535*^9, 3.591438538477868*^9}, {
  3.591438744063892*^9, 3.5914387593877497`*^9}, {3.5914387971392183`*^9, 
  3.5914388125411777`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Tests on Algebras", "Section",
 CellChangeTimes->{{3.547530963085421*^9, 3.547530965499281*^9}, {
  3.547533539488453*^9, 3.547533542206141*^9}}],

Cell[CellGroupData[{

Cell["Property Tests", "Subsection",
 CellChangeTimes->{{3.547530963085421*^9, 3.547530965499281*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Self", "-", "conjugacy"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.54753368627308*^9, 3.547533690088892*^9}, {
  3.547565633353714*^9, 3.547565635583506*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDSelfConj", "[", "x_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"CDConj", "[", "x", "]"}], " ", "-", " ", "x"}], " ", "//", " ", 
   "Expand"}]}]], "Input",
 CellChangeTimes->{{3.547533691207966*^9, 3.547533699057453*^9}, {
  3.5475337547885847`*^9, 3.547533756602049*^9}, {3.547565623442501*^9, 
  3.5475656280953083`*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", "Commutator", " ", "*)"}]], "Input",
 CellChangeTimes->{{3.547532706285275*^9, 3.547532709198914*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDComm", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"CDMult", "[", 
     RowBox[{"x", ",", "y"}], "]"}], " ", "-", " ", 
    RowBox[{"CDMult", "[", 
     RowBox[{"y", ",", "x"}], "]"}]}], " ", "//", " ", "Expand"}]}]], "Input",\

 CellChangeTimes->{{3.547532710801659*^9, 3.5475327236960583`*^9}, {
  3.5475327815956707`*^9, 3.547532786528801*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", "Associator", " ", "*)"}]], "Input",
 CellChangeTimes->{{3.54753279505739*^9, 3.547532798151845*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDAssoc", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"CDMult", "[", 
     RowBox[{
      RowBox[{"CDMult", "[", 
       RowBox[{"x", ",", "y"}], "]"}], ",", "z"}], "]"}], " ", "-", " ", 
    RowBox[{"CDMult", "[", 
     RowBox[{"x", ",", 
      RowBox[{"CDMult", "[", 
       RowBox[{"y", ",", "z"}], "]"}]}], "]"}]}], " ", "//", " ", 
   "Expand"}]}]], "Input",
 CellChangeTimes->{{3.547532802902133*^9, 3.547532827570059*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Alternator", " ", "--"}], " ", "for", " ", "testing", " ", 
   "alternative", " ", 
   RowBox[{"(", 
    RowBox[{"2", "-", 
     RowBox[{"variable", " ", "associative"}]}], ")"}], " ", "property"}], 
  " ", "*)"}]], "Input",
 CellChangeTimes->{{3.547532830552931*^9, 3.5475328552078667`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDAlter", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"CDAssoc", "[", 
      RowBox[{"x", ",", "x", ",", "y"}], "]"}], ",", 
     RowBox[{"CDAssoc", "[", 
      RowBox[{"x", ",", "y", ",", "x"}], "]"}], ",", 
     RowBox[{"CDAssoc", "[", 
      RowBox[{"y", ",", "x", ",", "x"}], "]"}]}], "}"}], " ", "//", " ", 
   "Expand"}]}]], "Input",
 CellChangeTimes->{{3.547532856831482*^9, 3.547532922237091*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Norm", "-", 
   RowBox[{"multiplicativtiy", " ", "property"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.547532944398703*^9, 3.547532956449553*^9}, {
  3.547533008116632*^9, 3.5475330150702257`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDNormMult", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"CDNorm", "[", 
     RowBox[{"CDMult", "[", 
      RowBox[{"x", ",", "y"}], "]"}], "]"}], " ", "-", " ", 
    RowBox[{
     RowBox[{"CDNorm", "[", "x", "]"}], "*", 
     RowBox[{"CDNorm", "[", "y", "]"}]}]}], " ", "//", " ", 
   "Expand"}]}]], "Input",
 CellChangeTimes->{{3.5475329584667177`*^9, 3.5475329827150373`*^9}, {
  3.547533017076758*^9, 3.5475330189392757`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["0 - 1 - Real numbers", "Subsection",
 CellChangeTimes->{{3.5475656790162573`*^9, 3.5475657099424543`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"CDMult", "[", 
    RowBox[{
     RowBox[{"Array", "[", 
      RowBox[{"a", ",", "1"}], "]"}], ",", 
     RowBox[{"Array", "[", 
      RowBox[{"b", ",", "1"}], "]"}]}], "]"}], " ", "-", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", "[", "1", "]"}], "*", 
     RowBox[{"b", "[", "1", "]"}]}], "}"}]}], " ", "//", " ", 
  "Expand"}]], "Input",
 CellChangeTimes->{{3.54756224444281*^9, 3.547562273078884*^9}, {
  3.5475627273735332`*^9, 3.547562728939171*^9}}],

Cell[BoxData[
 RowBox[{"CDSelfConj", "[", 
  RowBox[{"Array", "[", 
   RowBox[{"a", ",", "1"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.547533678227415*^9, 3.547533678968471*^9}, {
   3.547533736713375*^9, 3.547533749334013*^9}, 3.5475656597881937`*^9}],

Cell[BoxData[
 RowBox[{"CDComm", "[", 
  RowBox[{
   RowBox[{"Array", "[", 
    RowBox[{"a", ",", "1"}], "]"}], ",", 
   RowBox[{"Array", "[", 
    RowBox[{"b", ",", "1"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.547533620350965*^9, 3.547533640456193*^9}}],

Cell[BoxData[
 RowBox[{"CDAssoc", "[", 
  RowBox[{
   RowBox[{"Array", "[", 
    RowBox[{"a", ",", "1"}], "]"}], ",", 
   RowBox[{"Array", "[", 
    RowBox[{"b", ",", "1"}], "]"}], ",", 
   RowBox[{"Array", "[", 
    RowBox[{"c", ",", "1"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.547533642949203*^9, 3.5475336602095547`*^9}}],

Cell[BoxData[
 RowBox[{"CDNormMult", "[", 
  RowBox[{
   RowBox[{"Array", "[", 
    RowBox[{"a", ",", "1"}], "]"}], ",", 
   RowBox[{"Array", "[", 
    RowBox[{"b", ",", "1"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5475336655819683`*^9, 3.547533673297038*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"All", " ", "properties", " ", "are", " ", 
    RowBox[{"present", ":", " ", "real"}]}], ",", " ", "commuting", ",", " ", 
   "associative", ",", " ", 
   RowBox[{
    RowBox[{"norm", " ", "of", " ", "product"}], " ", "=", " ", 
    RowBox[{"product", " ", "of", " ", "norms"}]}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.620253726758589*^9, 3.6202537317459784`*^9}, {
  3.6202538150331793`*^9, 3.6202538409129333`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["1 - 2 - Complex numbers", "Subsection",
 CellChangeTimes->{{3.547565693703135*^9, 3.547565704086217*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"makecmplx", "[", "x_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "I"}], "}"}], ".", "x"}]}]], "Input",
 CellChangeTimes->{{3.547562755496325*^9, 3.547562764404645*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"makecmplx", "[", 
    RowBox[{"CDMult", "[", 
     RowBox[{
      RowBox[{"Array", "[", 
       RowBox[{"a", ",", "2"}], "]"}], ",", 
      RowBox[{"Array", "[", 
       RowBox[{"b", ",", "2"}], "]"}]}], "]"}], "]"}], " ", "-", " ", 
   RowBox[{
    RowBox[{"makecmplx", "[", 
     RowBox[{"Array", "[", 
      RowBox[{"a", ",", "2"}], "]"}], "]"}], "*", 
    RowBox[{"makecmplx", "[", 
     RowBox[{"Array", "[", 
      RowBox[{"b", ",", "2"}], "]"}], "]"}]}]}], " ", "//", " ", 
  "Expand"}]], "Input",
 CellChangeTimes->{{3.5475627182954397`*^9, 3.5475627921799307`*^9}}],

Cell[BoxData[
 RowBox[{"CDSelfConj", "[", 
  RowBox[{"{", 
   RowBox[{"0", ",", "1"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.547533788175952*^9, 3.5475337923235703`*^9}, 
   3.547565663434309*^9}],

Cell[BoxData[
 RowBox[{"CDComm", "[", 
  RowBox[{
   RowBox[{"Array", "[", 
    RowBox[{"a", ",", "2"}], "]"}], ",", 
   RowBox[{"Array", "[", 
    RowBox[{"b", ",", "2"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.547533620350965*^9, 3.547533640456193*^9}, {
  3.547533812077527*^9, 3.5475338144912167`*^9}}],

Cell[BoxData[
 RowBox[{"CDAssoc", "[", 
  RowBox[{
   RowBox[{"Array", "[", 
    RowBox[{"a", ",", "2"}], "]"}], ",", 
   RowBox[{"Array", "[", 
    RowBox[{"b", ",", "2"}], "]"}], ",", 
   RowBox[{"Array", "[", 
    RowBox[{"c", ",", "2"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.547533642949203*^9, 3.5475336602095547`*^9}, {
  3.54753381744479*^9, 3.547533823835389*^9}}],

Cell[BoxData[
 RowBox[{"CDNormMult", "[", 
  RowBox[{
   RowBox[{"Array", "[", 
    RowBox[{"a", ",", "2"}], "]"}], ",", 
   RowBox[{"Array", "[", 
    RowBox[{"b", ",", "2"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5475336655819683`*^9, 3.547533673297038*^9}, {
  3.5475338274441032`*^9, 3.547533830451342*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Property", " ", 
   RowBox[{"lost", ":", " ", "reality"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.6202538495497103`*^9, 3.620253857088853*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["2 - 4 - Quaternions", "Subsection",
 CellChangeTimes->{{3.5475657204321537`*^9, 3.54756572473437*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"quattopauli", "[", "x_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"x", "[", 
     RowBox[{"[", "1", "]"}], "]"}], "*", 
    RowBox[{"IdentityMatrix", "[", "2", "]"}]}], " ", "-", " ", 
   RowBox[{"I", "*", 
    RowBox[{"Sum", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"x", "[", 
        RowBox[{"[", 
         RowBox[{"k", "+", "1"}], "]"}], "]"}], "*", 
       RowBox[{"PauliMatrix", "[", "k", "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"k", ",", "3"}], "}"}]}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.547562800391358*^9, 3.547562830768888*^9}, {
  3.547562903045909*^9, 3.547562906293844*^9}, {3.5475907486105957`*^9, 
  3.5475907506426353`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"quattopauli", "[", 
    RowBox[{"CDMult", "[", 
     RowBox[{
      RowBox[{"Array", "[", 
       RowBox[{"a", ",", "4"}], "]"}], ",", 
      RowBox[{"Array", "[", 
       RowBox[{"b", ",", "4"}], "]"}]}], "]"}], "]"}], " ", "-", " ", 
   RowBox[{
    RowBox[{"quattopauli", "[", 
     RowBox[{"Array", "[", 
      RowBox[{"a", ",", "4"}], "]"}], "]"}], ".", 
    RowBox[{"quattopauli", "[", 
     RowBox[{"Array", "[", 
      RowBox[{"b", ",", "4"}], "]"}], "]"}]}]}], " ", "//", " ", 
  "Expand"}]], "Input",
 CellChangeTimes->{{3.5475628342641706`*^9, 3.547562893268825*^9}, 
   3.5475907543381243`*^9, {3.554435063390132*^9, 3.554435064747487*^9}}],

Cell[BoxData[
 RowBox[{"CDComm", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "1", ",", "0"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.547533620350965*^9, 3.547533640456193*^9}, {
  3.547533812077527*^9, 3.5475338144912167`*^9}, {3.5475339380229197`*^9, 
  3.5475339420282497`*^9}}],

Cell[BoxData[
 RowBox[{"CDAssoc", "[", 
  RowBox[{
   RowBox[{"Array", "[", 
    RowBox[{"a", ",", "4"}], "]"}], ",", 
   RowBox[{"Array", "[", 
    RowBox[{"b", ",", "4"}], "]"}], ",", 
   RowBox[{"Array", "[", 
    RowBox[{"c", ",", "4"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.547533642949203*^9, 3.5475336602095547`*^9}, {
  3.54753381744479*^9, 3.547533823835389*^9}, {3.547533947279529*^9, 
  3.547533952980719*^9}}],

Cell[BoxData[
 RowBox[{"CDNormMult", "[", 
  RowBox[{
   RowBox[{"Array", "[", 
    RowBox[{"a", ",", "4"}], "]"}], ",", 
   RowBox[{"Array", "[", 
    RowBox[{"b", ",", "4"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5475336655819683`*^9, 3.547533673297038*^9}, {
  3.5475338274441032`*^9, 3.547533830451342*^9}, {3.54753395725593*^9, 
  3.547533959628509*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Property", " ", 
   RowBox[{"lost", ":", " ", "commutation"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.620253891599172*^9, 3.620253897209285*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["3 - 8 - Octonions", "Subsection",
 CellChangeTimes->{{3.5475657204321537`*^9, 3.5475657521506453`*^9}}],

Cell[BoxData[
 RowBox[{"CDAssoc", "[", 
  RowBox[{
   RowBox[{"PadRight", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}], ",", "8"}], "]"}], ",", 
   RowBox[{"PadRight", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "1"}], "}"}], ",", "8"}], "]"}], ",", 
   RowBox[{"PadRight", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "0", ",", "0", ",", "1"}], "}"}], ",", 
     "8"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5475339715427094`*^9, 3.5475339997177896`*^9}}],

Cell[BoxData[
 RowBox[{"CDAlter", "[", 
  RowBox[{
   RowBox[{"Array", "[", 
    RowBox[{"a", ",", "8"}], "]"}], ",", 
   RowBox[{"Array", "[", 
    RowBox[{"b", ",", "8"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.547533642949203*^9, 3.5475336602095547`*^9}, {
  3.54753381744479*^9, 3.547533823835389*^9}, {3.547533947279529*^9, 
  3.547533952980719*^9}, {3.547534020742432*^9, 3.547534030949195*^9}}],

Cell[BoxData[
 RowBox[{"CDNormMult", "[", 
  RowBox[{
   RowBox[{"Array", "[", 
    RowBox[{"a", ",", "8"}], "]"}], ",", 
   RowBox[{"Array", "[", 
    RowBox[{"b", ",", "8"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5475336655819683`*^9, 3.547533673297038*^9}, {
  3.5475338274441032`*^9, 3.547533830451342*^9}, {3.54753395725593*^9, 
  3.547533959628509*^9}, {3.547534013607733*^9, 3.5475340168531933`*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Property", " ", 
    RowBox[{"lost", ":", " ", 
     RowBox[{"general", " ", "association"}]}]}], ",", " ", 
   RowBox[{"though", " ", "alternation", " ", 
    RowBox[{"(", 
     RowBox[{
     "association", " ", "with", " ", "two", " ", "inputs", " ", "equal"}], 
     ")"}], " ", "is", " ", "still", " ", "present"}]}], " ", "*)"}]], "Input",\

 CellChangeTimes->{{3.62025392899325*^9, 3.6202539603381023`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["4 - 16 - Sedenions", "Subsection",
 CellChangeTimes->{{3.5475657204321537`*^9, 3.54756574551061*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Example", " ", "from", " ", "the", " ", "Wikipedia", " ", "article", " ", 
   "on", " ", "them"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.5475620614610157`*^9, 3.5475620684839697`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"svvec", "[", 
   RowBox[{"n_", ",", "k_"}], "]"}], " ", ":=", " ", 
  RowBox[{"PadRight", "[", 
   RowBox[{
    RowBox[{"PadLeft", "[", 
     RowBox[{
      RowBox[{"{", "1", "}"}], ",", "k"}], "]"}], ",", "n"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.547534681099146*^9, 3.547534706437386*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sdx1", " ", "=", " ", 
   RowBox[{
    RowBox[{"svvec", "[", 
     RowBox[{"16", ",", "3"}], "]"}], "+", 
    RowBox[{"svvec", "[", 
     RowBox[{"16", ",", "10"}], "]"}]}]}], ";", " ", 
  RowBox[{"sdx2", " ", "=", " ", 
   RowBox[{
    RowBox[{"svvec", "[", 
     RowBox[{"16", ",", "6"}], "]"}], "-", 
    RowBox[{"svvec", "[", 
     RowBox[{"16", ",", "15"}], "]"}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5475348797527323`*^9, 3.547534914943204*^9}}],

Cell[BoxData[
 RowBox[{"CDNorm", " ", "/@", " ", 
  RowBox[{"{", 
   RowBox[{"sdx1", ",", "sdx2"}], "}"}]}]], "Input",
 CellChangeTimes->{{3.5475349242812023`*^9, 3.547534930206841*^9}}],

Cell[BoxData[
 RowBox[{"CDMult", "[", 
  RowBox[{"sdx1", ",", "sdx2"}], "]"}]], "Input",
 CellChangeTimes->{{3.547534818363277*^9, 3.547534851230444*^9}, 
   3.547534888792411*^9, {3.547534933151957*^9, 3.547534936181944*^9}}],

Cell[BoxData[
 RowBox[{"CDAlter", "[", 
  RowBox[{"sdx1", ",", "sdx2"}], "]"}]], "Input",
 CellChangeTimes->{{3.547534818363277*^9, 3.5475348667692966`*^9}, {
  3.5475349406326647`*^9, 3.547534943238346*^9}}],

Cell[BoxData[
 RowBox[{"CDNormMult", "[", 
  RowBox[{"sdx1", ",", "sdx2"}], "]"}]], "Input",
 CellChangeTimes->{{3.54753494646148*^9, 3.547534951952333*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Properties", " ", 
    RowBox[{"lost", ":", " ", "alternation"}]}], ",", " ", 
   RowBox[{
    RowBox[{"norm", " ", "of", " ", "product"}], " ", "=", " ", 
    RowBox[{"product", " ", "of", " ", 
     RowBox[{"norms", ".", " ", "Property"}], " ", 
     RowBox[{"gained", ":", " ", 
      RowBox[{
      "zero", " ", "getting", " ", "nonzero", " ", "divisors"}]}]}]}]}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.6202539908962717`*^9, 3.620254041213462*^9}}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Automorphism Symmetry Groups", "Section",
 CellChangeTimes->{{3.547530963085421*^9, 3.547530965499281*^9}, {
   3.5475332490375977`*^9, 3.54753325589084*^9}, 3.547566742435484*^9, {
   3.620255986067173*^9, 3.620255988223508*^9}}],

Cell[CellGroupData[{

Cell["General Symmetry Matrices", "Subsection",
 CellChangeTimes->{{3.5911858425594997`*^9, 3.5911858657553053`*^9}, {
  3.591186046949532*^9, 3.5911860482851057`*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{
    RowBox[{
    "Test", " ", "if", " ", "a", " ", "matrix", " ", "is", " ", "an", " ", 
     RowBox[{"automorphism", ".", " ", "The"}], " ", "matrix", " ", "is", " ",
      "square", " ", "with", " ", "dimension", " ", 
     RowBox[{"(", 
      RowBox[{"n", "-", 
       RowBox[{"ion", " ", "dimensions"}]}], ")"}]}], " ", "-", " ", 
    RowBox[{
    "1.", " ", "It", " ", "is", " ", "an", " ", "automorphism", " ", "of", 
     " ", "the", " ", "\"\<imaginary\>\"", " ", "part"}]}], ",", " ", 
   RowBox[{
   "since", " ", "automorphisms", " ", "do", " ", "not", " ", "affect", " ", 
    "the", " ", "\"\<real\>\"", " ", 
    RowBox[{"part", "."}]}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.6202058642221212`*^9, 3.620205872023131*^9}, {
  3.620206806105626*^9, 3.6202068456101217`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDAutoTest0", "[", "mat_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "mtab", ",", "vecs", ",", "v1", ",", "v2", ",", "va1", ",", "va2", ",", 
      "mvx1", ",", "mvx2", ",", "v12", ",", "mvx12"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"mtab", " ", "=", " ", 
      RowBox[{"CDUnitMultTab", "[", 
       RowBox[{"Log", "[", 
        RowBox[{"2", ",", 
         RowBox[{
          RowBox[{"Length", "[", "mat", "]"}], "+", "1"}]}], "]"}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"vecs", " ", "=", " ", 
      RowBox[{"IdentityMatrix", "[", 
       RowBox[{"Length", "[", "mat", "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"va1", " ", "=", " ", 
         RowBox[{"Prepend", "[", 
          RowBox[{"v1", ",", "0"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"va2", " ", "=", " ", 
         RowBox[{"Prepend", "[", 
          RowBox[{"v2", ",", "0"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"mvx1", " ", "=", " ", 
         RowBox[{"Prepend", "[", 
          RowBox[{
           RowBox[{"mat", ".", "v1"}], ",", "0"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"mvx2", " ", "=", " ", 
         RowBox[{"Prepend", "[", 
          RowBox[{
           RowBox[{"mat", ".", "v2"}], ",", "0"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"v12", " ", "=", " ", 
         RowBox[{"MultWithTab", "[", 
          RowBox[{"mtab", ",", "va1", ",", "va2"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"mvx12", " ", "=", " ", 
         RowBox[{"Prepend", "[", 
          RowBox[{
           RowBox[{"mat", ".", 
            RowBox[{"Rest", "[", "v12", "]"}]}], ",", 
           RowBox[{"First", "[", "v12", "]"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"MultWithTab", "[", 
          RowBox[{"mtab", ",", "mvx1", ",", "mvx2"}], "]"}], " ", "-", " ", 
         "mvx12"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"v1", ",", "vecs"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"v2", ",", "vecs"}], "}"}]}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.620205877910019*^9, 3.620205943278216*^9}, {
   3.62020599127488*^9, 3.620206026001581*^9}, {3.62020606814881*^9, 
   3.620206088642589*^9}, {3.620206244382792*^9, 3.620206358164027*^9}, {
   3.6202066397873993`*^9, 3.620206664612708*^9}, 3.620206716556278*^9, {
   3.620224959872513*^9, 3.6202249876626263`*^9}, 3.6206720515128593`*^9, {
   3.6206728840119057`*^9, 3.6206728874828444`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"AddMatRealPart", "[", 
   RowBox[{"rrel_", ",", "mat_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "n", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"n", " ", "=", " ", 
      RowBox[{"Length", "[", "mat", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"SparseArray", "[", 
      RowBox[{"ArrayFlatten", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"{", "rrel", "}"}], "}"}], ",", 
           RowBox[{"ConstantArray", "[", 
            RowBox[{"0", ",", 
             RowBox[{"{", 
              RowBox[{"1", ",", "n"}], "}"}]}], "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"ConstantArray", "[", 
            RowBox[{"0", ",", 
             RowBox[{"{", 
              RowBox[{"n", ",", "1"}], "}"}]}], "]"}], ",", "mat"}], "}"}]}], 
        "}"}], "]"}], "]"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.6206723556208963`*^9, 3.620672435195681*^9}, {
  3.6206731358292017`*^9, 3.6206731427719727`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDAutoTest", "[", "mat_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"matx", ",", "mtab"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"matx", " ", "=", " ", 
      RowBox[{"AddMatRealPart", "[", 
       RowBox[{"1", ",", 
        RowBox[{"Transpose", "[", "mat", "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"mtab", " ", "=", " ", 
      RowBox[{"CDUnitMultTab", "[", 
       RowBox[{"Log", "[", 
        RowBox[{"2", ",", 
         RowBox[{
          RowBox[{"Length", "[", "mat", "]"}], "+", "1"}]}], "]"}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"matx", ".", 
       RowBox[{"Transpose", "[", 
        RowBox[{"matx", ".", 
         RowBox[{"Transpose", "[", "mtab", "]"}]}], "]"}]}], " ", "-", " ", 
      RowBox[{"mtab", ".", "matx"}]}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.620672044625472*^9, 3.620672218083099*^9}, {
  3.620672271577888*^9, 3.620672274905571*^9}, {3.620672405883691*^9, 
  3.620672419498938*^9}, {3.620672551277313*^9, 3.620672647412691*^9}, {
  3.6206728635867767`*^9, 3.620672896052926*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Complex", " ", "numbers"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.6202585635035877`*^9, 3.620258572693508*^9}}],

Cell[BoxData[
 RowBox[{"CDAutoTest", "[", 
  RowBox[{"{", 
   RowBox[{"{", "r", "}"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.620257348470812*^9, 3.620257354695909*^9}}],

Cell[BoxData[
 RowBox[{"Reduce", "[", 
  RowBox[{"Thread", "[", 
   RowBox[{
    RowBox[{"Flatten", "[", "%", "]"}], "\[Equal]", "0"}], "]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.620257609276124*^9, 3.620257615338778*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Group", ":", " ", 
    RowBox[{
     RowBox[{"Z2", " ", "--"}], " ", "identity"}]}], ",", " ", 
   RowBox[{"complex", " ", "conjugate"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.620258575860654*^9, 3.620258586093912*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", "Quaternions", " ", "*)"}]], "Input",
 CellChangeTimes->{{3.620258611811022*^9, 3.62025861430206*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"ASSymbol", "[", "n_", "]"}], " ", ":=", " ", 
  RowBox[{"SparseArray", "[", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"p", "\[Rule]", 
      RowBox[{"Signature", "[", "p", "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"p", ",", 
       RowBox[{"Permutations", "[", 
        RowBox[{"Range", "[", "n", "]"}], "]"}]}], "}"}]}], "]"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.620258459084671*^9, 3.62025849650909*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"qvec", " ", "=", " ", 
   RowBox[{"Array", "[", 
    RowBox[{"q", ",", "4", ",", "0"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.620259251818371*^9, 3.620259265862089*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"quatatmmat", " ", "=", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"First", "[", "qvec", "]"}], "^", "2"}], "-", 
         RowBox[{
          RowBox[{"Rest", "[", "qvec", "]"}], ".", 
          RowBox[{"Rest", "[", "qvec", "]"}]}]}], ")"}], "*", 
       RowBox[{"IdentityMatrix", "[", "3", "]"}]}], "+", 
      RowBox[{"2", "*", 
       RowBox[{"Outer", "[", 
        RowBox[{"Times", ",", 
         RowBox[{"Rest", "[", "qvec", "]"}], ",", 
         RowBox[{"Rest", "[", "qvec", "]"}]}], "]"}]}], "+", 
      RowBox[{"2", "*", 
       RowBox[{"First", "[", "qvec", "]"}], "*", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"ASSymbol", "[", "3", "]"}], ".", 
         RowBox[{"Rest", "[", "qvec", "]"}]}], ")"}]}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"qvec", ".", "qvec"}], ")"}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.620259419779106*^9, 3.620259469408162*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"CDAutoTest", "[", "quatatmmat", "]"}], " ", "//", " ", "Factor"}],
   " ", "//", " ", "SparseArray"}]], "Input",
 CellChangeTimes->{{3.6202573596394*^9, 3.62025752273071*^9}, {
   3.620257590235139*^9, 3.620257592614809*^9}, 3.620258674787014*^9, {
   3.620259296014879*^9, 3.620259357049912*^9}, {3.620259399674452*^9, 
   3.620259433614765*^9}, {3.620259477584063*^9, 3.6202594798967333`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"quatatmmat", ".", 
     RowBox[{"Transpose", "[", "quatatmmat", "]"}]}], "-", 
    RowBox[{"IdentityMatrix", "[", "3", "]"}]}], " ", "//", " ", "Factor"}], 
  " ", "//", " ", "SparseArray"}]], "Input",
 CellChangeTimes->{{3.620259494001668*^9, 3.620259514578936*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Det", "[", "quatatmmat", "]"}], " ", "//", " ", 
  "Factor"}]], "Input",
 CellChangeTimes->{{3.620259518326*^9, 3.620259525953636*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"SO", 
   RowBox[{"(", "3", ")"}], " ", "matrices"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.6202595301393843`*^9, 3.620259533440913*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Continuous: Lie Algebras", "Subsection",
 CellChangeTimes->{{3.5475665092721367`*^9, 3.547566511974283*^9}, {
  3.620257638165307*^9, 3.620257642273994*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Test", " ", "on", " ", "a", " ", "departure", " ", "of", " ", "a", " ", 
   "matrix", " ", "from", " ", "the", " ", "identity", " ", "matrix"}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.620206503631158*^9, 3.620206514646356*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDDiffAutoTest0", "[", "mat_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "mtab", ",", "vecs", ",", "v1", ",", "v2", ",", "va1", ",", "va2", ",", 
      "mvx1", ",", "mvx2", ",", "v12", ",", "mvx12"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"mtab", " ", "=", " ", 
      RowBox[{"CDUnitMultTab", "[", 
       RowBox[{"Log", "[", 
        RowBox[{"2", ",", 
         RowBox[{
          RowBox[{"Length", "[", "mat", "]"}], "+", "1"}]}], "]"}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"vecs", " ", "=", " ", 
      RowBox[{"IdentityMatrix", "[", 
       RowBox[{"Length", "[", "mat", "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"va1", " ", "=", " ", 
         RowBox[{"Prepend", "[", 
          RowBox[{"v1", ",", "0"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"va2", " ", "=", " ", 
         RowBox[{"Prepend", "[", 
          RowBox[{"v2", ",", "0"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"mvx1", " ", "=", " ", 
         RowBox[{"Prepend", "[", 
          RowBox[{
           RowBox[{"mat", ".", "v1"}], ",", "0"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"mvx2", " ", "=", " ", 
         RowBox[{"Prepend", "[", 
          RowBox[{
           RowBox[{"mat", ".", "v2"}], ",", "0"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"v12", " ", "=", " ", 
         RowBox[{"MultWithTab", "[", 
          RowBox[{"mtab", ",", "va1", ",", "va2"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"mvx12", " ", "=", " ", 
         RowBox[{"Prepend", "[", 
          RowBox[{
           RowBox[{"mat", ".", 
            RowBox[{"Rest", "[", "v12", "]"}]}], ",", "0"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"MultWithTab", "[", 
          RowBox[{"mtab", ",", "mvx1", ",", "va2"}], "]"}], "+", 
         RowBox[{"MultWithTab", "[", 
          RowBox[{"mtab", ",", "va1", ",", "mvx2"}], "]"}], " ", "-", " ", 
         "mvx12"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"v1", ",", "vecs"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"v2", ",", "vecs"}], "}"}]}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.620206523615404*^9, 3.620206524647375*^9}, {
   3.620206576553977*^9, 3.620206596371451*^9}, {3.620206708751122*^9, 
   3.620206742906135*^9}, {3.620224858855628*^9, 3.620224887093709*^9}, {
   3.6202249189770107`*^9, 3.620224935563881*^9}, 3.6206729662219677`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"CDDiffAutoTest", "[", "mat_", "]"}], " ", ":=", " ", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"matx", ",", "mtab"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"matx", " ", "=", " ", 
       RowBox[{"AddMatRealPart", "[", 
        RowBox[{"0", ",", 
         RowBox[{"Transpose", "[", "mat", "]"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"mtab", " ", "=", " ", 
       RowBox[{"CDUnitMultTab", "[", 
        RowBox[{"Log", "[", 
         RowBox[{"2", ",", 
          RowBox[{
           RowBox[{"Length", "[", "mat", "]"}], "+", "1"}]}], "]"}], "]"}]}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"matx", ".", "mtab"}], " ", "+", " ", 
       RowBox[{"Transpose", "[", 
        RowBox[{"matx", ".", 
         RowBox[{"Transpose", "[", "mtab", "]"}]}], "]"}], " ", "-", " ", 
       RowBox[{"mtab", ".", "matx"}]}]}]}], "]"}]}], 
  "\[IndentingNewLine]"}]], "Input",
 CellChangeTimes->{{3.6206729697727203`*^9, 3.6206729749283524`*^9}, {
  3.620673035547121*^9, 3.6206730957415037`*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Quaternions", ":", " ", 
   RowBox[{"test", " ", "of", " ", "SO", 
    RowBox[{"(", "3", ")"}], " ", "algebra"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.620288668449897*^9, 3.620288676486408*^9}}],

Cell[BoxData[
 RowBox[{"SparseArray", "[", 
  RowBox[{"CDDiffAutoTest", " ", "/@", " ", 
   RowBox[{"ASSymbol", "[", "3", "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6202886520885677`*^9, 3.6202886828479233`*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Find", " ", "algebra", " ", "for", " ", "general", " ", "n"}], 
   "-", "ions"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.620288744321404*^9, 3.620288785487606*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"AMAlgebra", "[", "np_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "n", ",", "nx", ",", "i", ",", "j", ",", "r", ",", "vars", ",", "mat", 
      ",", "sol", ",", "matsol", ",", "matlist"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"n", " ", "=", " ", 
      RowBox[{"2", "^", "np"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"nx", " ", "=", " ", 
      RowBox[{"n", " ", "-", " ", "1"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"vars", " ", "=", " ", 
      RowBox[{"Flatten", "[", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"r", "[", 
          RowBox[{"i", ",", "j"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "n"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"j", ",", 
           RowBox[{"i", "+", "1"}], ",", "n"}], "}"}]}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"mat", " ", "=", " ", 
      RowBox[{"Array", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Signature", "[", 
           RowBox[{"{", "##", "}"}], "]"}], "*", 
          RowBox[{"(", 
           RowBox[{"r", " ", "@@", " ", 
            RowBox[{"Sort", "[", 
             RowBox[{"{", "##", "}"}], "]"}]}], ")"}]}], "&"}], ",", 
        RowBox[{"{", 
         RowBox[{"nx", ",", "nx"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"sol", " ", "=", " ", 
      RowBox[{"ToRules", "[", 
       RowBox[{"Reduce", "[", 
        RowBox[{"Thread", "[", 
         RowBox[{
          RowBox[{"Flatten", "[", 
           RowBox[{"CDDiffAutoTest", "[", "mat", "]"}], "]"}], "\[Equal]", 
          "0"}], "]"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"matsol", " ", "=", " ", 
      RowBox[{"mat", " ", "/.", " ", "sol"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"matlist", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"SparseArray", "[", 
         RowBox[{"D", "[", 
          RowBox[{"matsol", ",", "#"}], "]"}], "]"}], "&"}], " ", "/@", " ", 
       "vars"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"SparseArray", "[", 
      RowBox[{"Select", "[", 
       RowBox[{"matlist", ",", 
        RowBox[{
         RowBox[{
          RowBox[{"Max", "[", 
           RowBox[{"Abs", "[", 
            RowBox[{"Flatten", "[", 
             RowBox[{"Normal", "[", "#", "]"}], "]"}], "]"}], "]"}], ">", 
          "0"}], "&"}]}], "]"}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.620288797932783*^9, 3.620288972746867*^9}, {
  3.620289022900011*^9, 3.620289161487164*^9}, {3.620289855663373*^9, 
  3.62028997275001*^9}, {3.620290006142325*^9, 3.620290039556942*^9}, {
  3.620290071009156*^9, 3.620290072608218*^9}, {3.62147024800483*^9, 
  3.621470289682128*^9}, {3.621470326321794*^9, 3.6214703579612703`*^9}, {
  3.62150287363643*^9, 3.621502877738043*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Continuous", " ", "symmetry", " ", 
    RowBox[{"groups", ":", "\[IndentingNewLine]", 
     RowBox[{"0", " ", "-", " ", "none"}]}]}], ",", " ", 
   RowBox[{"1", " ", "-", " ", "none"}], ",", " ", 
   RowBox[{"2", " ", "-", " ", 
    RowBox[{"SO", 
     RowBox[{"(", "3", ")"}]}]}], ",", " ", "3", ",", "4", ",", "5", ",", 
   RowBox[{"6", " ", "-", " ", "G2"}], ",", " ", 
   RowBox[{"the", " ", "rest", " ", 
    RowBox[{"G2", "?"}]}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.620291118895173*^9, 3.62029114568183*^9}, {
  3.620673963385758*^9, 3.620673969563841*^9}, {3.620674290592144*^9, 
  3.620674290910492*^9}}],

Cell[BoxData[
 RowBox[{"AMAlg2", " ", "=", " ", 
  RowBox[{"AMAlgebra", "[", "2", "]"}]}]], "Input",
 CellChangeTimes->{{3.6202889773217287`*^9, 3.620288979316456*^9}, {
  3.620290056053041*^9, 3.620290092153871*^9}, {3.621469573374559*^9, 
  3.621469591801282*^9}}],

Cell[BoxData[
 RowBox[{"AMAlg3", " ", "=", " ", 
  RowBox[{"AMAlgebra", "[", "3", "]"}]}]], "Input",
 CellChangeTimes->{{3.6202889773217287`*^9, 3.620288979316456*^9}, {
  3.620290056053041*^9, 3.620290092153871*^9}, {3.621469597228483*^9, 
  3.621469602667693*^9}}],

Cell[BoxData[
 RowBox[{"AMAlg4", " ", "=", " ", 
  RowBox[{"AMAlgebra", "[", "4", "]"}]}]], "Input",
 CellChangeTimes->{{3.621469604867668*^9, 3.621469610292808*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", "Commutator", " ", "*)"}]], "Input",
 CellChangeTimes->{{3.547567026735015*^9, 3.547567029589123*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"commut", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"x", ".", "y"}], " ", "-", " ", 
   RowBox[{"y", ".", "x"}]}]}]], "Input",
 CellChangeTimes->{{3.459360038981777*^9, 3.459360045331067*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Commutation", " ", "table"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.620291163115642*^9, 3.620291166617406*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CommutTable", "[", "alg_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"algf", ",", "condmat", ",", "projmat", ",", "m1", ",", "m2"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"algf", " ", "=", " ", 
      RowBox[{"SparseArray", "[", 
       RowBox[{"Flatten", " ", "/@", " ", "alg"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"condmat", " ", "=", " ", 
      RowBox[{"algf", ".", 
       RowBox[{"Transpose", "[", "algf", "]"}]}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"projmat", " ", "=", " ", 
      RowBox[{"SparseArray", "[", 
       RowBox[{
        RowBox[{"Inverse", "[", "condmat", "]"}], ".", "algf"}], "]"}]}], ";",
      "\[IndentingNewLine]", 
     RowBox[{"SparseArray", "[", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"SparseArray", "[", 
         RowBox[{"projmat", ".", 
          RowBox[{"Flatten", "[", 
           RowBox[{"commut", "[", 
            RowBox[{"m1", ",", "m2"}], "]"}], "]"}]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"m1", ",", "alg"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"m2", ",", "alg"}], "}"}]}], "]"}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.620290109507374*^9, 3.620290205089733*^9}, {
  3.620290250033928*^9, 3.620290316683997*^9}, {3.620290368570662*^9, 
  3.6202904434591084`*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Killing", " ", 
   RowBox[{"form", ":", " ", 
    RowBox[{"algebra", " ", "metric"}]}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.620290628099676*^9, 3.620290635596266*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"AlgMetric", "[", "comm_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"n", ",", "cm1", ",", "cm2"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"n", " ", "=", " ", 
      RowBox[{"Length", "[", "comm", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"SparseArray", "[", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"Tr", "[", 
         RowBox[{"cm1", ".", "cm2"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"cm1", ",", "comm"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"cm2", ",", "comm"}], "}"}]}], "]"}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.459360246466545*^9, 3.459360306760935*^9}, {
  3.459361048433702*^9, 3.4593610511683064`*^9}, {3.6202908204510527`*^9, 
  3.6202908327033167`*^9}, {3.620290900995611*^9, 3.620290906873745*^9}, {
  3.620290978570291*^9, 3.620291032527569*^9}, {3.6202912152418756`*^9, 
  3.620291235995309*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Test", " ", "for", " ", 
   RowBox[{"reducibility", ":", " ", 
    RowBox[{
    "what", " ", "matrices", " ", "commute", " ", "with", " ", "the", " ", 
     "algebra", " ", 
     RowBox[{"matrices", "?"}]}]}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.620291473189736*^9, 3.620291489323711*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CommutMats", "[", "alg_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "dims", ",", "n", ",", "cmat", ",", "c", ",", "vars", ",", "sol", ",", 
      "cmsol", ",", "cmts"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"dims", " ", "=", " ", 
      RowBox[{"Dimensions", "[", "alg", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"n", " ", "=", " ", 
      RowBox[{"dims", "[", 
       RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"cmat", " ", "=", " ", 
      RowBox[{"Array", "[", 
       RowBox[{"c", ",", 
        RowBox[{"{", 
         RowBox[{"n", ",", "n"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"vars", " ", "=", " ", 
      RowBox[{"Flatten", "[", "cmat", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"sol", " ", "=", " ", 
      RowBox[{"ToRules", "[", 
       RowBox[{"Reduce", "[", 
        RowBox[{"Thread", "[", 
         RowBox[{
          RowBox[{"Flatten", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"SparseArray", "[", 
              RowBox[{"commut", "[", 
               RowBox[{"cmat", ",", "#"}], "]"}], "]"}], "&"}], " ", "/@", 
            " ", "alg"}], "]"}], "\[Equal]", "0"}], "]"}], "]"}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"cmsol", " ", "=", " ", 
      RowBox[{"cmat", " ", "/.", " ", "sol"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"cmts", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"SparseArray", "[", 
         RowBox[{"D", "[", 
          RowBox[{"cmsol", ",", "#"}], "]"}], "]"}], "&"}], " ", "/@", " ", 
       "vars"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Select", "[", 
      RowBox[{"cmts", ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Max", "[", 
          RowBox[{"Abs", "[", 
           RowBox[{"Flatten", "[", "#", "]"}], "]"}], "]"}], ">", "0"}], 
        "&"}]}], "]"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.620291494039481*^9, 3.620291669012185*^9}, {
  3.620291722012312*^9, 3.620291788435389*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Discrete: Group of Units", "Subsection",
 CellChangeTimes->{{3.547566525336318*^9, 3.547566527214409*^9}, {
  3.547586393220335*^9, 3.5475863955674257`*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Find", " ", "automorphisms", " ", "of", " ", "absolute"}], "-", 
   RowBox[{"value", " ", 
    RowBox[{"table", " ", "--"}], " ", "PSL", 
    RowBox[{"(", 
     RowBox[{"2", ",", "n"}], ")"}]}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.459391469804446*^9, 3.45939147115301*^9}, {
  3.4593915080423737`*^9, 3.459391520098648*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"AMAbsUnitMats", "[", "np_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "pwrtwos", ",", "bdgs", ",", "vals", ",", "pmbases", ",", "pb", ",", 
      "tgtvals", ",", "tv"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"pwrtwos", " ", "=", " ", 
      RowBox[{"Reverse", "[", 
       RowBox[{"NestList", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"2", "*", "#"}], "&"}], ",", "1", ",", 
         RowBox[{"np", "-", "1"}]}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Use", " ", "nonzero", " ", "ones"}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"bdgs", " ", "=", " ", 
      RowBox[{"Rest", "[", 
       RowBox[{"MakeBinDigits", "[", "np", "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"vals", " ", "=", " ", 
      RowBox[{"bdgs", ".", "pwrtwos"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"pmbases", " ", "=", " ", 
      RowBox[{"Flatten", "[", 
       RowBox[{
        RowBox[{"Permutations", " ", "/@", " ", 
         RowBox[{"Subsets", "[", 
          RowBox[{"bdgs", ",", 
           RowBox[{"{", "np", "}"}]}], "]"}]}], ",", "1"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"tgtvals", " ", "=", " ", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Mod", "[", 
          RowBox[{
           RowBox[{"bdgs", ".", "pb"}], ",", "2"}], "]"}], ".", "pwrtwos"}], 
        ",", 
        RowBox[{"{", 
         RowBox[{"pb", ",", "pmbases"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"tgtvals", " ", "=", " ", 
      RowBox[{"Select", "[", 
       RowBox[{"tgtvals", ",", 
        RowBox[{
         RowBox[{
          RowBox[{"Min", "[", "#", "]"}], ">", "0"}], "&"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"SparseArray", "[", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"SparseArray", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"#", "\[Rule]", "1"}], "&"}], " ", "/@", " ", 
          RowBox[{"Transpose", "[", 
           RowBox[{"{", 
            RowBox[{"vals", ",", "tv"}], "}"}], "]"}]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"tv", ",", "tgtvals"}], "}"}]}], "]"}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.620292776236637*^9, 3.620292805510621*^9}, {
  3.6202930663715887`*^9, 3.620293086922173*^9}, {3.620294163239079*^9, 
  3.620294251117324*^9}, {3.620294484482624*^9, 3.620294549100379*^9}, {
  3.62029459278621*^9, 3.620294752548428*^9}, {3.620296341267684*^9, 
  3.620296342101075*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDAbsAutoTest0", "[", "mat_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "mtab", ",", "vecs", ",", "v1", ",", "v2", ",", "va1", ",", "va2", ",", 
      "mvx1", ",", "mvx2", ",", "v12", ",", "mvx12"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"mtab", " ", "=", " ", 
      RowBox[{"Abs", "[", 
       RowBox[{"CDUnitMultTab", "[", 
        RowBox[{"Log", "[", 
         RowBox[{"2", ",", 
          RowBox[{
           RowBox[{"Length", "[", "mat", "]"}], "+", "1"}]}], "]"}], "]"}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"vecs", " ", "=", " ", 
      RowBox[{"IdentityMatrix", "[", 
       RowBox[{"Length", "[", "mat", "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"va1", " ", "=", " ", 
         RowBox[{"Prepend", "[", 
          RowBox[{"v1", ",", "0"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"va2", " ", "=", " ", 
         RowBox[{"Prepend", "[", 
          RowBox[{"v2", ",", "0"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"mvx1", " ", "=", " ", 
         RowBox[{"Prepend", "[", 
          RowBox[{
           RowBox[{"mat", ".", "v1"}], ",", "0"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"mvx2", " ", "=", " ", 
         RowBox[{"Prepend", "[", 
          RowBox[{
           RowBox[{"mat", ".", "v2"}], ",", "0"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"v12", " ", "=", " ", 
         RowBox[{"MultWithTab", "[", 
          RowBox[{"mtab", ",", "va1", ",", "va2"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"mvx12", " ", "=", " ", 
         RowBox[{"Prepend", "[", 
          RowBox[{
           RowBox[{"mat", ".", 
            RowBox[{"Rest", "[", "v12", "]"}]}], ",", 
           RowBox[{"First", "[", "v12", "]"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"MultWithTab", "[", 
          RowBox[{"mtab", ",", "mvx1", ",", "mvx2"}], "]"}], " ", "-", " ", 
         "mvx12"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"v1", ",", "vecs"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"v2", ",", "vecs"}], "}"}]}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.620205877910019*^9, 3.620205943278216*^9}, {
   3.62020599127488*^9, 3.620206026001581*^9}, {3.62020606814881*^9, 
   3.620206088642589*^9}, {3.620206244382792*^9, 3.620206358164027*^9}, {
   3.6202066397873993`*^9, 3.620206664612708*^9}, 3.620206716556278*^9, {
   3.620224959872513*^9, 3.6202249876626263`*^9}, 3.6206720515128593`*^9, {
   3.6206728840119057`*^9, 3.6206728874828444`*^9}, {3.620675744914461*^9, 
   3.620675765581058*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDAbsAutoTest", "[", "mat_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"matx", ",", "mtab"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"matx", " ", "=", " ", 
      RowBox[{"AddMatRealPart", "[", 
       RowBox[{"1", ",", 
        RowBox[{"Transpose", "[", "mat", "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"mtab", " ", "=", " ", 
      RowBox[{"Abs", "[", 
       RowBox[{"CDUnitMultTab", "[", 
        RowBox[{"Log", "[", 
         RowBox[{"2", ",", 
          RowBox[{
           RowBox[{"Length", "[", "mat", "]"}], "+", "1"}]}], "]"}], "]"}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"matx", ".", 
       RowBox[{"Transpose", "[", 
        RowBox[{"matx", ".", 
         RowBox[{"Transpose", "[", "mtab", "]"}]}], "]"}]}], " ", "-", " ", 
      RowBox[{"mtab", ".", "matx"}]}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.620672044625472*^9, 3.620672218083099*^9}, {
  3.620672271577888*^9, 3.620672274905571*^9}, {3.620672405883691*^9, 
  3.620672419498938*^9}, {3.620672551277313*^9, 3.620672647412691*^9}, {
  3.6206728635867767`*^9, 3.620672896052926*^9}, {3.6206757430101767`*^9, 
  3.62067577736985*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"AMAbsUnitMatsVerify", "[", "np_", "]"}], " ", ":=", " ", 
  RowBox[{"SparseArray", "[", 
   RowBox[{"CDAbsAutoTest", " ", "/@", " ", 
    RowBox[{"AMAbsUnitMats", "[", "np", "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6202948017882032`*^9, 3.6202948267076693`*^9}, 
   3.620296357817072*^9, {3.620298425902548*^9, 3.6202984277983418`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"AMAbsUnitMats", "[", "1", "]"}], " ", "//", " ", 
  "Normal"}]], "Input",
 CellChangeTimes->{{3.620298431493194*^9, 3.620298448780357*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Sort", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"AMAbsUnitMats", "[", "2", "]"}], "]"}], "]"}], " ", "===", " ", 
  RowBox[{"Sort", "[", 
   RowBox[{"Permutations", "[", 
    RowBox[{"IdentityMatrix", "[", "3", "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.620298467724352*^9, 3.620298487452477*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Add", " ", "signs"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.620295066898525*^9, 3.620295069437693*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"AMUnitMats1", "[", "np_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"absmats", ",", "sgns", ",", "amt", ",", "sn"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"absmats", " ", "=", " ", 
      RowBox[{"AMAbsUnitMats", "[", "np", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"sgns", " ", "=", " ", 
      RowBox[{"Tuples", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"1", ",", 
          RowBox[{"-", "1"}]}], "}"}], ",", 
        RowBox[{
         RowBox[{"2", "^", "np"}], "-", "1"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"SparseArray", "[", 
      RowBox[{"Flatten", "[", 
       RowBox[{
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"Select", "[", 
           RowBox[{
            RowBox[{"Table", "[", 
             RowBox[{
              RowBox[{"SparseArray", "[", 
               RowBox[{"sn", "*", "amt"}], "]"}], ",", 
              RowBox[{"{", 
               RowBox[{"sn", ",", "sgns"}], "}"}]}], "]"}], ",", 
            RowBox[{
             RowBox[{
              RowBox[{"Max", "[", 
               RowBox[{"Abs", "[", 
                RowBox[{"Flatten", "[", 
                 RowBox[{"CDAutoTest", "[", "#", "]"}], "]"}], "]"}], "]"}], 
              "\[Equal]", "0"}], "&"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"amt", ",", "absmats"}], "}"}]}], "]"}], ",", "1"}], "]"}],
       "]"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.620295999277294*^9, 3.6202960520593*^9}, {
   3.620296096662401*^9, 3.620296316518857*^9}, 3.620296360129526*^9, 
   3.620298176188294*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"AMUnitMats2", "[", "np_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "absmats", ",", "sgns", ",", "s", ",", "amt", ",", "sgnamt", ",", 
      "sol"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"absmats", " ", "=", " ", 
      RowBox[{"AMAbsUnitMats", "[", "np", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"sgns", " ", "=", " ", 
      RowBox[{"Array", "[", 
       RowBox[{"s", ",", 
        RowBox[{
         RowBox[{"2", "^", "np"}], "-", "1"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"SparseArray", "[", 
      RowBox[{"Flatten", "[", 
       RowBox[{
        RowBox[{"Table", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"sgnamt", " ", "=", " ", 
            RowBox[{"SparseArray", "[", 
             RowBox[{"sgns", "*", "amt"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"sol", " ", "=", " ", 
            RowBox[{"{", 
             RowBox[{"ToRules", "[", 
              RowBox[{"Reduce", "[", 
               RowBox[{"Thread", "[", 
                RowBox[{
                 RowBox[{"Flatten", "[", 
                  RowBox[{"CDAutoTest", "[", "sgnamt", "]"}], "]"}], 
                 "\[Equal]", "0"}], "]"}], "]"}], "]"}], "}"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"SparseArray", "[", 
              RowBox[{"#", "*", "amt"}], "]"}], "&"}], " ", "/@", " ", 
            RowBox[{"(", 
             RowBox[{"sgns", " ", "//.", " ", "sol"}], ")"}]}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"amt", ",", "absmats"}], "}"}]}], "]"}], ",", "1"}], "]"}],
       "]"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.620295999277294*^9, 3.6202960520593*^9}, {
   3.620296096662401*^9, 3.620296316518857*^9}, 3.620296360129526*^9, {
   3.620297459547279*^9, 3.620297487707759*^9}, {3.62029751988124*^9, 
   3.620297713723836*^9}, 3.620298178662654*^9}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Diagonal", ":", " ", 
    RowBox[{"sign", " ", "at", " ", "index", " ", "ix"}], ":", " ", 
    RowBox[{"s", 
     RowBox[{"(", "ix", ")"}]}]}], " ", "=", " ", 
   RowBox[{"product", " ", "of", " ", "s", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"2", "^", "i"}], ")"}], "^", "b"}], 
    RowBox[{"(", "i", ")"}], " ", "where", " ", "s", 
    RowBox[{"(", 
     RowBox[{"2", "^", "i"}], ")"}], " ", "are", " ", "some", " ", "input", 
    " ", "signs", " ", "and", " ", "b", 
    RowBox[{"(", "i", ")"}], " ", "is", " ", "the", " ", "bits", " ", "of", 
    " ", "ix"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.620775251375523*^9, 3.6207752913940487`*^9}, {
  3.6207903618053493`*^9, 3.620790396519034*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"AMUnitDiagonals", "[", "np_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"bds", ",", "b1", ",", "b2"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"bds", " ", "=", " ", 
      RowBox[{"MakeBinDigits", "[", "np", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Rest", " ", "/@", " ", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "^", 
         RowBox[{"(", 
          RowBox[{"b1", ".", "b2"}], ")"}]}], ",", 
        RowBox[{"{", 
         RowBox[{"b1", ",", "bds"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"b2", ",", "bds"}], "}"}]}], "]"}]}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.620775232418355*^9, 3.620775248090028*^9}, {
  3.620790516245913*^9, 3.6207905669613857`*^9}, {3.620790959937014*^9, 
  3.620790962021556*^9}, {3.6207910925074*^9, 3.620791095548267*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"AMUnitMats3", "[", "np_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "absmats", ",", "sgnsrc", ",", "k", ",", "sgns", ",", "amt", ",", "sn", 
      ",", "itmats", ",", "imt", ",", "dgss", ",", "ds"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"absmats", " ", "=", " ", 
      RowBox[{"AMAbsUnitMats", "[", "np", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"sgnsrc", " ", "=", " ", 
      RowBox[{"ConstantArray", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"1", ",", 
          RowBox[{"-", "1"}]}], "}"}], ",", 
        RowBox[{
         RowBox[{"2", "^", "np"}], "-", "1"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"sgnsrc", "[", 
         RowBox[{"[", 
          RowBox[{"2", "^", "k"}], "]"}], "]"}], " ", "=", " ", 
        RowBox[{"{", "1", "}"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", "0", ",", 
         RowBox[{"np", "-", "1"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"sgns", " ", "=", " ", 
      RowBox[{"Tuples", "[", "sgnsrc", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"itmats", " ", "=", " ", 
      RowBox[{"Flatten", "[", 
       RowBox[{
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"Select", "[", 
           RowBox[{
            RowBox[{"Table", "[", 
             RowBox[{
              RowBox[{"SparseArray", "[", 
               RowBox[{"sn", "*", "amt"}], "]"}], ",", 
              RowBox[{"{", 
               RowBox[{"sn", ",", "sgns"}], "}"}]}], "]"}], ",", 
            RowBox[{
             RowBox[{
              RowBox[{"Max", "[", 
               RowBox[{"Abs", "[", 
                RowBox[{"Flatten", "[", 
                 RowBox[{"CDAutoTest", "[", "#", "]"}], "]"}], "]"}], "]"}], 
              "\[Equal]", "0"}], "&"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"amt", ",", "absmats"}], "}"}]}], "]"}], ",", "1"}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"dgss", " ", "=", " ", 
      RowBox[{"AMUnitDiagonals", "[", "np", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"SparseArray", "[", 
      RowBox[{"Flatten", "[", 
       RowBox[{
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"SparseArray", "[", 
           RowBox[{"ds", "*", "imt"}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"imt", ",", "itmats"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"ds", ",", "dgss"}], "}"}]}], "]"}], ",", "1"}], "]"}], 
      "]"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.620295999277294*^9, 3.6202960520593*^9}, {
   3.620296096662401*^9, 3.620296316518857*^9}, 3.620296360129526*^9, 
   3.620298176188294*^9, {3.620791557984603*^9, 3.6207915583803673`*^9}, {
   3.620802239209043*^9, 3.620802283577917*^9}, {3.6208025282361298`*^9, 
   3.6208026477187634`*^9}}],

Cell[BoxData[
 RowBox[{"AMUnitMats3", "[", "2", "]"}]], "Input",
 CellChangeTimes->{{3.6208022888020897`*^9, 3.6208022950826597`*^9}, {
  3.6208026749840717`*^9, 3.6208026834452972`*^9}}],

Cell[BoxData[
 RowBox[{"AMUnitMats4", "[", "2", "]"}]], "Input",
 CellChangeTimes->{{3.6208022888020897`*^9, 3.6208022950826597`*^9}, {
  3.6208023633808517`*^9, 3.620802363701578*^9}, {3.620802808769553*^9, 
  3.620802810008287*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"AMUnitMats4", "[", "np_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "absmats", ",", "sgns", ",", "k", ",", "s", ",", "amt", ",", "sgnamt", 
      ",", "sol", ",", "itmats", ",", "imt", ",", "dgss", ",", "ds"}], "}"}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"absmats", " ", "=", " ", 
      RowBox[{"AMAbsUnitMats", "[", "np", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"sgns", " ", "=", " ", 
      RowBox[{"Array", "[", 
       RowBox[{"s", ",", 
        RowBox[{
         RowBox[{"2", "^", "np"}], "-", "1"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"sgns", "[", 
         RowBox[{"[", 
          RowBox[{"2", "^", "k"}], "]"}], "]"}], " ", "=", " ", "1"}], ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", "0", ",", 
         RowBox[{"np", "-", "1"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"itmats", " ", "=", " ", 
      RowBox[{"Flatten", "[", 
       RowBox[{
        RowBox[{"Table", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"sgnamt", " ", "=", " ", 
            RowBox[{"SparseArray", "[", 
             RowBox[{"sgns", "*", "amt"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"sol", " ", "=", " ", 
            RowBox[{"{", 
             RowBox[{"ToRules", "[", 
              RowBox[{"Reduce", "[", 
               RowBox[{"Thread", "[", 
                RowBox[{
                 RowBox[{"Flatten", "[", 
                  RowBox[{"CDAutoTest", "[", "sgnamt", "]"}], "]"}], 
                 "\[Equal]", "0"}], "]"}], "]"}], "]"}], "}"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"SparseArray", "[", 
              RowBox[{"#", "*", "amt"}], "]"}], "&"}], " ", "/@", " ", 
            RowBox[{"(", 
             RowBox[{"sgns", " ", "//.", " ", "sol"}], ")"}]}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"amt", ",", "absmats"}], "}"}]}], "]"}], ",", "1"}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"dgss", " ", "=", " ", 
      RowBox[{"AMUnitDiagonals", "[", "np", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"SparseArray", "[", 
      RowBox[{"Flatten", "[", 
       RowBox[{
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"SparseArray", "[", 
           RowBox[{"ds", "*", "imt"}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"imt", ",", "itmats"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"ds", ",", "dgss"}], "}"}]}], "]"}], ",", "1"}], "]"}], 
      "]"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.620295999277294*^9, 3.6202960520593*^9}, {
   3.620296096662401*^9, 3.620296316518857*^9}, 3.620296360129526*^9, {
   3.620297459547279*^9, 3.620297487707759*^9}, {3.62029751988124*^9, 
   3.620297713723836*^9}, 3.620298178662654*^9, {3.6207915611946487`*^9, 
   3.620791561404715*^9}, {3.620802339332931*^9, 3.620802352874651*^9}, {
   3.620802778024991*^9, 3.62080279631427*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"AMUnitMats1", "[", "1", "]"}], " ", "//", " ", "Normal"}], " ", "//",
   " ", "Sort"}]], "Input",
 CellChangeTimes->{{3.620298503353533*^9, 3.620298525357114*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"AMUnitMats2", "[", "1", "]"}], " ", "//", " ", "Normal"}], " ", "//",
   " ", "Sort"}]], "Input",
 CellChangeTimes->{{3.620298503353533*^9, 3.620298529301166*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"AMUnitMats3", "[", "1", "]"}], " ", "//", " ", "Normal"}], " ", "//",
   " ", "Sort"}]], "Input",
 CellChangeTimes->{{3.620298503353533*^9, 3.620298525357114*^9}, {
  3.620802737919024*^9, 3.620802738061743*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"AMUnitMats4", "[", "1", "]"}], " ", "//", " ", "Normal"}], " ", "//",
   " ", "Sort"}]], "Input",
 CellChangeTimes->{{3.620298503353533*^9, 3.620298529301166*^9}, {
  3.6208027413830957`*^9, 3.620802741581891*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"AMUnitMatsQuat", " ", "=", " ", 
   RowBox[{"Select", "[", 
    RowBox[{
     RowBox[{"Sort", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"#", "[", 
          RowBox[{"[", "1", "]"}], "]"}], "*", 
         RowBox[{"#", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], "&"}], " ", "/@", " ", 
       RowBox[{"Tuples", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Tuples", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{
              RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", "3"}], "]"}], ",", 
          RowBox[{"Permutations", "[", 
           RowBox[{"IdentityMatrix", "[", "3", "]"}], "]"}]}], "}"}], "]"}]}],
       "]"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"Det", "[", "#", "]"}], ">", "0"}], "&"}]}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.620791197403164*^9, 3.620791208065703*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Sort", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"AMUnitMats1", "[", "2", "]"}], "]"}], "]"}], " ", "===", " ", 
  "AMUnitMatsQuat"}]], "Input",
 CellChangeTimes->{{3.62029854183331*^9, 3.6202986774938498`*^9}, {
  3.620791218087484*^9, 3.620791218810721*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Sort", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"AMUnitMats2", "[", "2", "]"}], "]"}], "]"}], " ", "===", " ", 
  "AMUnitMatsQuat"}]], "Input",
 CellChangeTimes->{{3.62029854183331*^9, 3.6202986859816847`*^9}, 
   3.620791195921073*^9, 3.6207912263640747`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Sort", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"AMUnitMats3", "[", "2", "]"}], "]"}], "]"}], " ", "===", " ", 
  "AMUnitMatsQuat"}]], "Input",
 CellChangeTimes->{{3.62029854183331*^9, 3.6202986774938498`*^9}, {
  3.620791218087484*^9, 3.620791218810721*^9}, {3.620802717328917*^9, 
  3.620802717517704*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Sort", "[", 
   RowBox[{"Normal", "[", 
    RowBox[{"AMUnitMats4", "[", "2", "]"}], "]"}], "]"}], " ", "===", " ", 
  "AMUnitMatsQuat"}]], "Input",
 CellChangeTimes->{{3.62029854183331*^9, 3.6202986859816847`*^9}, 
   3.620791195921073*^9, 3.6207912263640747`*^9, {3.620802720335438*^9, 
   3.6208027205082006`*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Find", " ", "group", " ", "identity"}], ",", " ", "inverses", ",",
    " ", 
   RowBox[{"conjugacy", " ", "classes"}], ",", " ", "..."}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.459402286666212*^9, 3.459402302695282*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"FindIndexIn", "[", 
   RowBox[{"xlst_List", ",", "x_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"idx", "=", "0"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"xlst", "[", 
           RowBox[{"[", "i", "]"}], "]"}], " ", "===", " ", "x"}], ",", " ", 
         RowBox[{
          RowBox[{"idx", " ", "=", " ", "i"}], ";", " ", 
          RowBox[{"Break", "[", "]"}]}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{"Length", "[", "xlst", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", "idx"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.459409700578463*^9, 3.459409785095954*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"FindIndexOfSetIn", "[", 
   RowBox[{"xstlst_", ",", "x_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"idx", " ", "=", " ", "0"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"MemberQ", "[", 
          RowBox[{
           RowBox[{"xstlst", "[", 
            RowBox[{"[", "i", "]"}], "]"}], ",", "x"}], "]"}], ",", 
         RowBox[{
          RowBox[{"idx", " ", "=", " ", "i"}], ";", " ", 
          RowBox[{"Break", "[", "]"}]}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{"Length", "[", "xstlst", "]"}]}], "}"}]}], "]"}], ";", " ", 
     "idx"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.459411795358138*^9, 3.45941187709939*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Returns", ":", " ", 
    RowBox[{"order", " ", "of", " ", "group"}]}], ",", " ", 
   RowBox[{"index", " ", "of", " ", "identity", " ", "element"}], ",", " ", 
   RowBox[{"indexes", " ", "of", " ", "inverses"}], ",", " ", 
   RowBox[{"info", " ", "for", " ", "each", " ", 
    RowBox[{"class", ":"}]}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.459409376989813*^9, 3.459409403608663*^9}, {
  3.4594094528328238`*^9, 3.4594094563847322`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"FindClasses", "[", 
   RowBox[{"elems_", ",", "mult_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "n", ",", "idfnd", ",", "id", ",", "ident", ",", "invlst", ",", "elsrmn",
       ",", "classes", ",", "nclss", ",", "clsselms", ",", "elclss", ",", 
      "cel", ",", "celpwr", ",", "pwrlst", ",", "clsmlttab", ",", "cl1", ",", 
      "cl2", ",", "clns"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"n", " ", "=", " ", 
      RowBox[{"Length", "[", "elems", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Find", " ", "identity", " ", "element"}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"idfnd", " ", "=", " ", "True"}], ";", "\[IndentingNewLine]", 
        
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"mult", "[", 
              RowBox[{
               RowBox[{"elems", "[", 
                RowBox[{"[", "i", "]"}], "]"}], ",", 
               RowBox[{"elems", "[", 
                RowBox[{"[", "j", "]"}], "]"}]}], "]"}], "=!=", 
             RowBox[{"elems", "[", 
              RowBox[{"[", "j", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"idfnd", " ", "=", " ", "False"}], ";", " ", 
             RowBox[{"Break", "[", "]"}]}]}], "]"}], ",", " ", 
          RowBox[{"{", 
           RowBox[{"j", ",", "n"}], "}"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{"idfnd", ",", 
          RowBox[{
           RowBox[{"id", " ", "=", " ", "i"}], ";", " ", 
           RowBox[{"Break", "[", "]"}]}]}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "n"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"ident", " ", "=", " ", 
      RowBox[{"elems", "[", 
       RowBox[{"[", "id", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Find", " ", "inverses"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"invlst", " ", "=", " ", 
      RowBox[{"Table", "[", 
       RowBox[{"0", ",", 
        RowBox[{"{", "n", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"Do", "[", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"mult", "[", 
             RowBox[{
              RowBox[{"elems", "[", 
               RowBox[{"[", "i", "]"}], "]"}], ",", 
              RowBox[{"elems", "[", 
               RowBox[{"[", "j", "]"}], "]"}]}], "]"}], "===", "ident"}], ",",
            "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"invlst", "[", 
              RowBox[{"[", "i", "]"}], "]"}], " ", "=", " ", "j"}], ";", " ", 
            
            RowBox[{"Break", "[", "]"}]}]}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"j", ",", "n"}], "}"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "n"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Find", " ", "class", " ", "contents"}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"classes", " ", "=", " ", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"clsselms", " ", "=", " ", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"elsrmn", " ", "=", " ", "elems"}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "elsrmn", "]"}], " ", ">", " ", "0"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"elclss", " ", "=", " ", 
         RowBox[{"Union", "[", 
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{"mult", "[", 
             RowBox[{
              RowBox[{"mult", "[", 
               RowBox[{
                RowBox[{"elems", "[", 
                 RowBox[{"[", "i", "]"}], "]"}], ",", 
                RowBox[{"elsrmn", "[", 
                 RowBox[{"[", "1", "]"}], "]"}]}], "]"}], ",", 
              RowBox[{"elems", "[", 
               RowBox[{"[", 
                RowBox[{"invlst", "[", 
                 RowBox[{"[", "i", "]"}], "]"}], "]"}], "]"}]}], "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"i", ",", "n"}], "}"}]}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"elsrmn", " ", "=", " ", 
         RowBox[{"Complement", "[", 
          RowBox[{"elsrmn", ",", "elclss"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"classes", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Length", "[", "elclss", "]"}], ",", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"FindIndexIn", "[", 
                RowBox[{"elems", ",", "#"}], "]"}], "&"}], " ", "/@", " ", 
              "elclss"}], ")"}]}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"clsselms", ",", "elclss"}], "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"nclss", " ", "=", " ", 
      RowBox[{"Length", "[", "classes", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
      "Find", " ", "sequence", " ", "of", " ", "classes", " ", "of", " ", 
       "powers", " ", "of", " ", "an", " ", "element"}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"cel", " ", "=", " ", 
         RowBox[{"clsselms", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "1"}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"pwrlst", " ", "=", " ", 
         RowBox[{"{", "i", "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"celpwr", " ", "=", " ", "cel"}], ";", "\[IndentingNewLine]", 
        
        RowBox[{"While", "[", 
         RowBox[{
          RowBox[{"celpwr", " ", "=!=", " ", "ident"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"celpwr", " ", "=", " ", 
            RowBox[{"mult", "[", 
             RowBox[{"celpwr", ",", "cel"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"pwrlst", ",", 
             RowBox[{"FindIndexOfSetIn", "[", 
              RowBox[{"clsselms", ",", "celpwr"}], "]"}]}], "]"}]}]}], "]"}], 
        ";", "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{
          RowBox[{"classes", "[", 
           RowBox[{"[", "i", "]"}], "]"}], ",", 
          RowBox[{"Length", "[", "pwrlst", "]"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{
          RowBox[{"classes", "[", 
           RowBox[{"[", "i", "]"}], "]"}], ",", "pwrlst"}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "nclss"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Class", "-", 
       RowBox[{"multiplication", " ", "table"}]}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"clsmlttab", " ", "=", " ", 
      RowBox[{"Table", "[", 
       RowBox[{"0", ",", 
        RowBox[{"{", "nclss", "}"}], ",", 
        RowBox[{"{", "nclss", "}"}], ",", 
        RowBox[{"{", "nclss", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"cl1", " ", "=", " ", 
         RowBox[{"clsselms", "[", 
          RowBox[{"[", "i1", "]"}], "]"}]}], ";", " ", 
        RowBox[{"cl2", " ", "=", " ", 
         RowBox[{"clsselms", "[", 
          RowBox[{"[", "i2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"clsmlttab", "[", 
            RowBox[{"[", 
             RowBox[{"i1", ",", "i2", ",", 
              RowBox[{"FindIndexOfSetIn", "[", 
               RowBox[{"clsselms", ",", 
                RowBox[{"mult", "[", 
                 RowBox[{
                  RowBox[{"cl1", "[", 
                   RowBox[{"[", "j1", "]"}], "]"}], ",", 
                  RowBox[{"cl2", "[", 
                   RowBox[{"[", "j2", "]"}], "]"}]}], "]"}]}], "]"}]}], "]"}],
             "]"}], "++"}], ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"j1", ",", 
            RowBox[{"Length", "[", "cl1", "]"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"j2", ",", 
            RowBox[{"Length", "[", "cl2", "]"}]}], "}"}]}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"i1", ",", "nclss"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i2", ",", "nclss"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"clns", " ", "=", " ", 
      RowBox[{"Length", " ", "/@", " ", "clsselms"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"clsmlttab", "[", 
         RowBox[{"[", 
          RowBox[{"i", ",", "j"}], "]"}], "]"}], " ", "/=", " ", "clns"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "nclss"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "nclss"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "n", ",", "id", ",", "invlst", ",", "nclss", ",", "classes", ",", 
       "clsmlttab"}], "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.459408546964176*^9, 3.459408549112319*^9}, {
   3.459408777884499*^9, 3.459409065469489*^9}, {3.4594091152197104`*^9, 
   3.459409364603669*^9}, {3.459410018351468*^9, 3.459410304138069*^9}, {
   3.459410766143354*^9, 3.459410828359404*^9}, {3.459410867884897*^9, 
   3.459411000758692*^9}, {3.459411070995158*^9, 3.459411254796213*^9}, {
   3.459411701618462*^9, 3.459411761705305*^9}, {3.459411904556251*^9, 
   3.45941205629569*^9}, 3.459412115877283*^9, {3.459412301104617*^9, 
   3.459412524299184*^9}, {3.4594135022106857`*^9, 3.459413509247361*^9}, {
   3.4594136202586184`*^9, 3.459413644409348*^9}, {3.459425444257784*^9, 
   3.45942546652658*^9}, {3.45942553155967*^9, 3.459425581150414*^9}, {
   3.4594256342462883`*^9, 3.4594256450525723`*^9}, {3.459425894256073*^9, 
   3.459425903466222*^9}, {3.620294891699843*^9, 3.62029489352836*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"Groups", ":", "\[IndentingNewLine]", 
      RowBox[{"PSL", 
       RowBox[{"(", 
        RowBox[{"1", ",", "2"}], ")"}]}]}], " ", "=", " ", "identity"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"PSL", 
      RowBox[{"(", 
       RowBox[{"2", ",", "2"}], ")"}]}], " ", "=", " ", 
     RowBox[{
      RowBox[{"S", 
       RowBox[{"(", "3", ")"}]}], " ", "=", " ", 
      RowBox[{"D", 
       RowBox[{"(", "3", ")"}]}]}]}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"PSL", 
      RowBox[{"(", 
       RowBox[{"3", ",", "2"}], ")"}]}], " ", "=", " ", 
     RowBox[{"PSL", 
      RowBox[{"(", 
       RowBox[{"2", ",", "7"}], ")"}]}]}], ",", " ", 
    RowBox[{
     RowBox[{"PSL", 
      RowBox[{"(", 
       RowBox[{"4", ",", "2"}], ")"}]}], " ", "=", " ", 
     RowBox[{"A", 
      RowBox[{"(", "8", ")"}]}]}]}], " ", "*)"}], " "}]], "Input",
 CellChangeTimes->{{3.4594259344434347`*^9, 3.4594259574163103`*^9}, {
  3.459426038291523*^9, 3.459426079593214*^9}, {3.547586696030555*^9, 
  3.547586729707946*^9}, {3.5475868397943907`*^9, 3.547586851981992*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Groups", ":", " ", 
    RowBox[{"Z", 
     RowBox[{"(", "2", ")"}]}]}], ",", " ", 
   RowBox[{"S", 
    RowBox[{"(", "4", ")"}]}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.5475879589136868`*^9, 3.547587973785461*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Group", " ", "Character", " ", "Tables"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.459426145589799*^9, 3.45942614993788*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"charxor1", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", "1", "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"{", "1", "}"}], "}"}]}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.459426298177472*^9, 3.459426307764255*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"charxor2", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "1", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "1", ",", 
         RowBox[{"-", "1"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", 
         RowBox[{"-", "1"}], ",", "0"}], "}"}]}], "}"}]}], "}"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.45942627156336*^9, 3.459426294883936*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"charxor3", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "21", ",", "42", ",", "56", ",", "24", ",", "24"}], 
      "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1"}], 
        "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"6", ",", "2", ",", "0", ",", "0", ",", 
         RowBox[{"-", "1"}], ",", 
         RowBox[{"-", "1"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"7", ",", 
         RowBox[{"-", "1"}], ",", 
         RowBox[{"-", "1"}], ",", "1", ",", "0", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"8", ",", "0", ",", "0", ",", 
         RowBox[{"-", "1"}], ",", "1", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", 
         RowBox[{"-", "1"}], ",", "1", ",", "0", ",", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", 
            RowBox[{"I", "*", 
             RowBox[{"Sqrt", "[", "7", "]"}]}]}], ")"}], "/", "2"}], ",", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "-", 
            RowBox[{"I", "*", 
             RowBox[{"Sqrt", "[", "7", "]"}]}]}], ")"}], "/", "2"}]}], "}"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", 
         RowBox[{"-", "1"}], ",", "1", ",", "0", ",", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "-", 
            RowBox[{"I", "*", 
             RowBox[{"Sqrt", "[", "7", "]"}]}]}], ")"}], "/", "2"}], ",", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", 
            RowBox[{"I", "*", 
             RowBox[{"Sqrt", "[", "7", "]"}]}]}], ")"}], "/", "2"}]}], 
        "}"}]}], "}"}]}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.45942618290519*^9, 3.459426198224667*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"charxor4", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "1", ",", "105", ",", "210", ",", "1260", ",", "1120", ",", "2520", ",",
        "3360", ",", "2880", ",", "2880", ",", "112", ",", "1680", ",", 
       "1344", ",", "1344", ",", "1344"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
        "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", 
         "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1"}], 
        "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"7", ",", 
         RowBox[{"-", "1"}], ",", "3", ",", 
         RowBox[{"-", "1"}], ",", "1", ",", "1", ",", 
         RowBox[{"-", "1"}], ",", "0", ",", "0", ",", "4", ",", "0", ",", 
         RowBox[{"-", "1"}], ",", 
         RowBox[{"-", "1"}], ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
        "14", ",", "6", ",", "2", ",", "2", ",", "2", ",", "0", ",", "0", ",",
          "0", ",", "0", ",", 
         RowBox[{"-", "1"}], ",", 
         RowBox[{"-", "1"}], ",", 
         RowBox[{"-", "1"}], ",", 
         RowBox[{"-", "1"}], ",", 
         RowBox[{"-", "1"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"20", ",", "4", ",", "4", ",", "0", ",", 
         RowBox[{"-", "1"}], ",", "0", ",", "1", ",", 
         RowBox[{"-", "1"}], ",", 
         RowBox[{"-", "1"}], ",", "5", ",", "1", ",", "0", ",", "0", ",", 
         "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"21", ",", 
         RowBox[{"-", "3"}], ",", "1", ",", "1", ",", "0", ",", 
         RowBox[{"-", "1"}], ",", "0", ",", "0", ",", "0", ",", "6", ",", 
         RowBox[{"-", "2"}], ",", "1", ",", "1", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"21", ",", 
         RowBox[{"-", "3"}], ",", "1", ",", "1", ",", "0", ",", 
         RowBox[{"-", "1"}], ",", "0", ",", "0", ",", "0", ",", 
         RowBox[{"-", "3"}], ",", "1", ",", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", 
            RowBox[{"I", "*", 
             RowBox[{"Sqrt", "[", "15", "]"}]}]}], ")"}], "/", "2"}], ",", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "-", 
            RowBox[{"I", "*", 
             RowBox[{"Sqrt", "[", "15", "]"}]}]}], ")"}], "/", "2"}], ",", 
         "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"21", ",", 
         RowBox[{"-", "3"}], ",", "1", ",", "1", ",", "0", ",", 
         RowBox[{"-", "1"}], ",", "0", ",", "0", ",", "0", ",", 
         RowBox[{"-", "3"}], ",", "1", ",", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "-", 
            RowBox[{"I", "*", 
             RowBox[{"Sqrt", "[", "15", "]"}]}]}], ")"}], "/", "2"}], ",", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", 
            RowBox[{"I", "*", 
             RowBox[{"Sqrt", "[", "15", "]"}]}]}], ")"}], "/", "2"}], ",", 
         "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"28", ",", 
         RowBox[{"-", "4"}], ",", "4", ",", "0", ",", "1", ",", "0", ",", 
         RowBox[{"-", "1"}], ",", "0", ",", "0", ",", "1", ",", "1", ",", "1",
          ",", "1", ",", 
         RowBox[{"-", "2"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"35", ",", "3", ",", 
         RowBox[{"-", "5"}], ",", 
         RowBox[{"-", "1"}], ",", "2", ",", 
         RowBox[{"-", "1"}], ",", "0", ",", "0", ",", "0", ",", "5", ",", "1",
          ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"45", ",", 
         RowBox[{"-", "3"}], ",", 
         RowBox[{"-", "3"}], ",", "1", ",", "0", ",", "1", ",", "0", ",", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", 
            RowBox[{"I", "*", 
             RowBox[{"Sqrt", "[", "7", "]"}]}]}], ")"}], "/", "2"}], ",", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "-", 
            RowBox[{"I", "*", 
             RowBox[{"Sqrt", "[", "7", "]"}]}]}], ")"}], "/", "2"}], ",", "0",
          ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"45", ",", 
         RowBox[{"-", "3"}], ",", 
         RowBox[{"-", "3"}], ",", "1", ",", "0", ",", "1", ",", "0", ",", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "-", 
            RowBox[{"I", "*", 
             RowBox[{"Sqrt", "[", "7", "]"}]}]}], ")"}], "/", "2"}], ",", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", 
            RowBox[{"I", "*", 
             RowBox[{"Sqrt", "[", "7", "]"}]}]}], ")"}], "/", "2"}], ",", "0",
          ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"56", ",", "8", ",", "0", ",", "0", ",", 
         RowBox[{"-", "1"}], ",", "0", ",", 
         RowBox[{"-", "1"}], ",", "0", ",", "0", ",", 
         RowBox[{"-", "4"}], ",", "0", ",", "1", ",", "1", ",", "1"}], "}"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{"64", ",", "0", ",", "0", ",", "0", ",", 
         RowBox[{"-", "2"}], ",", "0", ",", "0", ",", "1", ",", "1", ",", "4",
          ",", "0", ",", 
         RowBox[{"-", "1"}], ",", 
         RowBox[{"-", "1"}], ",", 
         RowBox[{"-", "1"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"70", ",", 
         RowBox[{"-", "2"}], ",", "2", ",", 
         RowBox[{"-", "2"}], ",", "1", ",", "0", ",", "1", ",", "0", ",", "0",
          ",", 
         RowBox[{"-", "5"}], ",", 
         RowBox[{"-", "1"}], ",", "0", ",", "0", ",", "0"}], "}"}]}], "}"}]}],
     "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.459426241088663*^9, 3.459426243677512*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"charsgnxor1", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", 
         RowBox[{"-", "1"}]}], "}"}]}], "}"}]}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.459426326661908*^9, 3.459426338828483*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"charsgnxor2", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "3", ",", "8", ",", "6", ",", "6"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "1", ",", "1", ",", "1", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "1", ",", "1", ",", 
         RowBox[{"-", "1"}], ",", 
         RowBox[{"-", "1"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "2", ",", 
         RowBox[{"-", "1"}], ",", "0", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", 
         RowBox[{"-", "1"}], ",", "0", ",", "1", ",", 
         RowBox[{"-", "1"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", 
         RowBox[{"-", "1"}], ",", "0", ",", 
         RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "}"}]}], "}"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.459426314702726*^9, 3.459426323131791*^9}, {
  3.4594263599916267`*^9, 3.459426371109139*^9}, {3.459426506569268*^9, 
  3.459426508255538*^9}, {3.4594270716625032`*^9, 3.4594270835688753`*^9}, {
  3.459427134131422*^9, 3.45942714097766*^9}, {3.4594272347039537`*^9, 
  3.4594272590528383`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"charsgnxor3", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "1", ",", "7", ",", "42", ",", "42", ",", "84", ",", "168", ",", "224", 
       ",", "224", ",", "192", ",", "168", ",", "192"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
        "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", 
         "1", ",", "1", ",", "1", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
        "6", ",", "6", ",", "2", ",", "2", ",", "2", ",", "0", ",", "0", ",", 
         "0", ",", 
         RowBox[{"-", "1"}], ",", "0", ",", 
         RowBox[{"-", "1"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"7", ",", 
         RowBox[{"-", "1"}], ",", 
         RowBox[{"-", "1"}], ",", "3", ",", 
         RowBox[{"-", "1"}], ",", 
         RowBox[{"-", "1"}], ",", 
         RowBox[{"-", "1"}], ",", "1", ",", "0", ",", "1", ",", "0"}], "}"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{"7", ",", 
         RowBox[{"-", "1"}], ",", "3", ",", 
         RowBox[{"-", "1"}], ",", 
         RowBox[{"-", "1"}], ",", "1", ",", 
         RowBox[{"-", "1"}], ",", "1", ",", "0", ",", 
         RowBox[{"-", "1"}], ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"7", ",", "7", ",", 
         RowBox[{"-", "1"}], ",", 
         RowBox[{"-", "1"}], ",", 
         RowBox[{"-", "1"}], ",", 
         RowBox[{"-", "1"}], ",", "1", ",", "1", ",", "0", ",", 
         RowBox[{"-", "1"}], ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"8", ",", "8", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
         RowBox[{"-", "1"}], ",", 
         RowBox[{"-", "1"}], ",", "1", ",", "0", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"14", ",", 
         RowBox[{"-", "2"}], ",", "2", ",", "2", ",", 
         RowBox[{"-", "2"}], ",", "0", ",", "1", ",", 
         RowBox[{"-", "1"}], ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"21", ",", 
         RowBox[{"-", "3"}], ",", 
         RowBox[{"-", "3"}], ",", "1", ",", "1", ",", "1", ",", "0", ",", "0",
          ",", "0", ",", 
         RowBox[{"-", "1"}], ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"21", ",", 
         RowBox[{"-", "3"}], ",", "1", ",", 
         RowBox[{"-", "3"}], ",", "1", ",", 
         RowBox[{"-", "1"}], ",", "0", ",", "0", ",", "0", ",", "1", ",", 
         "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "3", ",", 
         RowBox[{"-", "1"}], ",", 
         RowBox[{"-", "1"}], ",", 
         RowBox[{"-", "1"}], ",", "1", ",", "0", ",", "0", ",", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", 
            RowBox[{"I", "*", 
             RowBox[{"Sqrt", "[", "7", "]"}]}]}], ")"}], "/", "2"}], ",", "1",
          ",", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "-", 
            RowBox[{"I", "*", 
             RowBox[{"Sqrt", "[", "7", "]"}]}]}], ")"}], "/", "2"}]}], "}"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "3", ",", 
         RowBox[{"-", "1"}], ",", 
         RowBox[{"-", "1"}], ",", 
         RowBox[{"-", "1"}], ",", "1", ",", "0", ",", "0", ",", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "-", 
            RowBox[{"I", "*", 
             RowBox[{"Sqrt", "[", "7", "]"}]}]}], ")"}], "/", "2"}], ",", "1",
          ",", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", 
            RowBox[{"I", "*", 
             RowBox[{"Sqrt", "[", "7", "]"}]}]}], ")"}], "/", "2"}]}], 
        "}"}]}], "}"}]}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.459426214137846*^9, 3.459426218642576*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Determinant Invariance", "Subsection",
 CellChangeTimes->{{3.591515198953595*^9, 3.591515235853236*^9}, {
  3.591533624902802*^9, 3.591533628116581*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Find", " ", "Hermitian", " ", "matrices", " ", "with", " ", "appropriate", 
   " ", "sizes"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.621468548570746*^9, 3.621468556718471*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Indices", " ", "in", " ", "upper", " ", "triangular", " ", "part", " ", 
   "of", " ", "matrix"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.621234640435874*^9, 3.621234653953971*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"MatIdx", "[", 
   RowBox[{"lbl_", ",", "n_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"ix", ",", "i", ",", "j"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ix", " ", "=", " ", "1"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"lbl", "[", 
         RowBox[{"i", ",", "j"}], "]"}], " ", "=", " ", 
        RowBox[{"ix", "++"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "n"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", 
         RowBox[{"i", "+", "1"}], ",", "n"}], "}"}]}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.6210492115766077`*^9, 3.621049271330057*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"How", " ", "many", " ", 
   RowBox[{"values", "?"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.621100371572571*^9, 3.621100375431259*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDHMVals", "[", 
   RowBox[{"m_", ",", "np_"}], "]"}], " ", ":=", " ", 
  RowBox[{"m", " ", "+", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "/", "2"}], ")"}], "m", 
      RowBox[{"(", 
       RowBox[{"m", "-", "1"}], ")"}]}], ")"}], "*", 
    RowBox[{"2", "^", "np"}]}]}]}]], "Input",
 CellChangeTimes->{{3.621100376976967*^9, 3.621100396671723*^9}, {
  3.621468567030705*^9, 3.62146856792036*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Off", "-", 
   RowBox[{
   "diagonal", " ", "with", " ", "conjugate", " ", "as", " ", 
    "appropriate"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.6212352082046633`*^9, 3.621235223607305*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDHOfDg", "[", 
   RowBox[{"m_", ",", "n_", ",", "x_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"ixs", ",", "ix", ",", "i", ",", "j"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"MatIdx", "[", 
      RowBox[{"ixs", ",", "m"}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Table", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Which", "[", 
        RowBox[{
         RowBox[{"j", ">", "i"}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"ix", " ", "=", " ", 
           RowBox[{"ixs", "[", 
            RowBox[{"i", ",", "j"}], "]"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"x", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"m", "+", 
              RowBox[{"n", "*", 
               RowBox[{"(", 
                RowBox[{"ix", "-", "1"}], ")"}]}], "+", "1"}], ";;", 
             RowBox[{"m", "+", 
              RowBox[{"n", "*", "ix"}]}]}], "]"}], "]"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"j", "<", "i"}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"ix", " ", "=", " ", 
           RowBox[{"ixs", "[", 
            RowBox[{"j", ",", "i"}], "]"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"CDConj", "[", 
           RowBox[{"x", "[", 
            RowBox[{"[", 
             RowBox[{
              RowBox[{"m", "+", 
               RowBox[{"n", "*", 
                RowBox[{"(", 
                 RowBox[{"ix", "-", "1"}], ")"}]}], "+", "1"}], ";;", 
              RowBox[{"m", "+", 
               RowBox[{"n", "*", "ix"}]}]}], "]"}], "]"}], "]"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"j", "\[Equal]", "i"}], ",", "\[IndentingNewLine]", "Null"}],
         "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"i", ",", "m"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "m"}], "}"}]}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.621234352689117*^9, 3.621234373021329*^9}, {
  3.621234405321403*^9, 3.621234423113681*^9}, {3.621234534584618*^9, 
  3.621234623002721*^9}, {3.6214686019763403`*^9, 3.6214686031639767`*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"For", " ", "a", " ", "m", "*", "m", " ", "matrix"}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.6210559676456747`*^9, 3.621055971428297*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDHMat", "[", 
   RowBox[{"m_", ",", "0", ",", "x_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"i", ",", "j", ",", "dg", ",", "od"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"dg", " ", "=", " ", 
      RowBox[{"Take", "[", 
       RowBox[{"x", ",", "m"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"od", " ", "=", " ", 
      RowBox[{"CDHOfDg", "[", 
       RowBox[{"m", ",", "1", ",", "x"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"j", " ", "\[Equal]", " ", "i"}], ",", 
         RowBox[{"dg", "[", 
          RowBox[{"[", "i", "]"}], "]"}], ",", 
         RowBox[{"First", "[", 
          RowBox[{"od", "[", 
           RowBox[{"[", 
            RowBox[{"i", ",", "j"}], "]"}], "]"}], "]"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "m"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "m"}], "}"}]}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.621055855134983*^9, 3.621055876497136*^9}, {
  3.6210559257834787`*^9, 3.621055932826901*^9}, {3.621055988595805*^9, 
  3.621056014541218*^9}, {3.6210999954907293`*^9, 3.62110006687096*^9}, {
  3.621100144772398*^9, 3.621100201927038*^9}, {3.621229928165283*^9, 
  3.621229943302824*^9}, {3.621229994065289*^9, 3.621229995352415*^9}, {
  3.621234707443048*^9, 3.621234789507728*^9}, {3.621468621809647*^9, 
  3.621468652643641*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDHMat", "[", 
   RowBox[{"m_", ",", "1", ",", "x_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"i", ",", "j", ",", "dg", ",", "od"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"dg", " ", "=", " ", 
      RowBox[{"Take", "[", 
       RowBox[{"x", ",", "m"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"od", " ", "=", " ", 
      RowBox[{"CDHOfDg", "[", 
       RowBox[{"m", ",", "2", ",", "x"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"j", " ", "\[Equal]", " ", "i"}], ",", 
         RowBox[{"dg", "[", 
          RowBox[{"[", "i", "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"1", ",", "I"}], "}"}], ".", 
          RowBox[{"od", "[", 
           RowBox[{"[", 
            RowBox[{"i", ",", "j"}], "]"}], "]"}]}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "m"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "m"}], "}"}]}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.621055855134983*^9, 3.621055876497136*^9}, {
  3.6210559257834787`*^9, 3.621055932826901*^9}, {3.621055988595805*^9, 
  3.621056014541218*^9}, {3.6210999954907293`*^9, 3.62110006687096*^9}, {
  3.621100144772398*^9, 3.621100201927038*^9}, {3.621229928165283*^9, 
  3.621229943302824*^9}, {3.621229994065289*^9, 3.621229995352415*^9}, {
  3.621234707443048*^9, 3.621234826299402*^9}, {3.621468626006351*^9, 
  3.6214686513884907`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDHMat", "[", 
   RowBox[{"m_", ",", "2", ",", "x_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"i", ",", "j", ",", "dg", ",", "od", ",", "id", ",", "qt"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"dg", " ", "=", " ", 
      RowBox[{"Take", "[", 
       RowBox[{"x", ",", "m"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"od", " ", "=", " ", 
      RowBox[{"CDHOfDg", "[", 
       RowBox[{"m", ",", "4", ",", "x"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"id", " ", "=", " ", 
      RowBox[{"IdentityMatrix", "[", "2", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"qt", " ", "=", " ", 
      RowBox[{"Prepend", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "I"}], "*", 
         RowBox[{"Array", "[", 
          RowBox[{"PauliMatrix", ",", "3"}], "]"}]}], ",", "id"}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"ArrayFlatten", "[", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"j", " ", "\[Equal]", " ", "i"}], ",", 
          RowBox[{
           RowBox[{"dg", "[", 
            RowBox[{"[", "i", "]"}], "]"}], "*", "id"}], ",", 
          RowBox[{
           RowBox[{"od", "[", 
            RowBox[{"[", 
             RowBox[{"i", ",", "j"}], "]"}], "]"}], ".", "qt"}]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "m"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"j", ",", "m"}], "}"}]}], "]"}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.621055855134983*^9, 3.621055876497136*^9}, {
   3.6210559257834787`*^9, 3.621055932826901*^9}, {3.621055988595805*^9, 
   3.621056014541218*^9}, {3.6210999954907293`*^9, 3.62110006687096*^9}, {
   3.621100144772398*^9, 3.621100201927038*^9}, {3.621229928165283*^9, 
   3.621229943302824*^9}, {3.621229994065289*^9, 3.621229995352415*^9}, {
   3.621234707443048*^9, 3.621234826299402*^9}, {3.621234931860561*^9, 
   3.621235021926858*^9}, 3.621237796748398*^9, {3.621468627747999*^9, 
   3.621468654220508*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Find", " ", "the", " ", "adjoint", " ", "of", " ", "a", " ", 
   RowBox[{"matrix", " ", "--"}], " ", "useful", " ", "for", " ", "finding", 
   " ", "variations", " ", "in", " ", "a", " ", 
   RowBox[{"matrix", "'"}], "s", " ", "determinant"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.621468864652549*^9, 3.621468881593583*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"MatrixAdjoint", "[", "mat_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{"Map", "[", 
    RowBox[{"Reverse", ",", 
     RowBox[{"Minors", "[", 
      RowBox[{
       RowBox[{"Transpose", "[", "mat", "]"}], ",", 
       RowBox[{
        RowBox[{"Length", "[", "mat", "]"}], "-", "1"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}]}], "]"}], 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "^", 
      RowBox[{"(", 
       RowBox[{"i", "+", "j"}], ")"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", "mat", "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", 
       RowBox[{"Length", "[", "mat", "]"}]}], "}"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.6214688974535027`*^9, 3.621468922758706*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"MatrixAdjointVerify", "[", "mat_", "]"}], " ", ":=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"MatrixAdjoint", "[", "mat", "]"}], " ", "-", " ", 
    RowBox[{
     RowBox[{"Det", "[", "mat", "]"}], "*", 
     RowBox[{"Inverse", "[", "mat", "]"}]}]}], " ", "//", " ", 
   "Factor"}]}]], "Input",
 CellChangeTimes->{{3.62146895015347*^9, 3.621468966268355*^9}, {
  3.621469008454418*^9, 3.621469009844726*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"grad", "[", 
   RowBox[{"f_", ",", "xvec_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{"f", ",", "x"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "xvec"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.621470751473922*^9, 3.621470771845993*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDHDetDiffAMAlgebra", "[", 
   RowBox[{"m_", ",", "np_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "nx", ",", "x", ",", "xvec", ",", "rmat", ",", "r", ",", "rvars", ",", 
      "hmat", ",", "hmdim", ",", "trval", ",", "trvexp", ",", "sol", ",", 
      "matsol", ",", "matlist"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"nx", " ", "=", " ", 
      RowBox[{"CDHMVals", "[", 
       RowBox[{"m", ",", "np"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"xvec", " ", "=", " ", 
      RowBox[{"Array", "[", 
       RowBox[{"x", ",", "nx"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"rmat", " ", "=", " ", 
      RowBox[{"Array", "[", 
       RowBox[{"r", ",", 
        RowBox[{"{", 
         RowBox[{"nx", ",", "nx"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"rvars", " ", "=", " ", 
      RowBox[{"Flatten", "[", "rmat", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"hmat", " ", "=", " ", 
      RowBox[{"CDHMat", "[", 
       RowBox[{"m", ",", "np", ",", "xvec"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"hmdim", " ", "=", " ", 
      RowBox[{"Length", "[", "hmat", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"trval", " ", "=", " ", 
      RowBox[{"Tr", "[", 
       RowBox[{
        RowBox[{"MatrixAdjoint", "[", "hmat", "]"}], ".", 
        RowBox[{"CDHMat", "[", 
         RowBox[{"m", ",", "np", ",", 
          RowBox[{"rmat", ".", "xvec"}]}], "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"trvexp", " ", "=", " ", 
      RowBox[{"Nest", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"SparseArray", "[", 
          RowBox[{"grad", "[", 
           RowBox[{"#", ",", "xvec"}], "]"}], "]"}], "&"}], ",", "trval", ",",
         "hmdim"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"sol", " ", "=", " ", 
      RowBox[{"ToRules", "[", 
       RowBox[{"Reduce", "[", 
        RowBox[{"Thread", "[", 
         RowBox[{
          RowBox[{"Flatten", "[", "trvexp", "]"}], "\[Equal]", "0"}], "]"}], 
        "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"matsol", " ", "=", " ", 
      RowBox[{"rmat", " ", "/.", " ", "sol"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"matlist", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"SparseArray", "[", 
         RowBox[{"D", "[", 
          RowBox[{"matsol", ",", "#"}], "]"}], "]"}], "&"}], " ", "/@", " ", 
       "rvars"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"SparseArray", "[", 
      RowBox[{"Select", "[", 
       RowBox[{"matlist", ",", 
        RowBox[{
         RowBox[{
          RowBox[{"Max", "[", 
           RowBox[{"Abs", "[", 
            RowBox[{"Flatten", "[", 
             RowBox[{"Normal", "[", "#", "]"}], "]"}], "]"}], "]"}], ">", 
          "0"}], "&"}]}], "]"}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.620288797932783*^9, 3.620288972746867*^9}, {
  3.620289022900011*^9, 3.620289161487164*^9}, {3.620289855663373*^9, 
  3.62028997275001*^9}, {3.620290006142325*^9, 3.620290039556942*^9}, {
  3.620290071009156*^9, 3.620290072608218*^9}, {3.62147024800483*^9, 
  3.621470289682128*^9}, {3.621470326321794*^9, 3.6214703579612703`*^9}, {
  3.621470439095028*^9, 3.621470743967546*^9}, {3.6214707753538237`*^9, 
  3.621470805737238*^9}, {3.6214710313566017`*^9, 3.6214711256512947`*^9}, {
  3.62149725894715*^9, 3.6214972596238813`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"GenDetDiffAMAlgebra", "[", 
   RowBox[{"detf_", ",", "nvarf_", ",", "nmatf_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "x", ",", "nx", ",", "xvec", ",", "mat", ",", "mdim", ",", "r", ",", 
      "rmat", ",", "rvars", ",", "dxv", ",", "dt", ",", "k", ",", "dfdt", ",",
       "nm", ",", "dfdtexp", ",", "sol", ",", "matsol", ",", "matlist"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"nx", " ", "=", " ", "nvarf"}], ";", "\[IndentingNewLine]", 
     RowBox[{"xvec", " ", "=", " ", 
      RowBox[{"Array", "[", 
       RowBox[{"x", ",", "nx"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"rmat", " ", "=", " ", 
      RowBox[{"Array", "[", 
       RowBox[{"r", ",", 
        RowBox[{"{", 
         RowBox[{"nx", ",", "nx"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"rvars", " ", "=", " ", 
      RowBox[{"Flatten", "[", "rmat", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"dxv", " ", "=", " ", 
      RowBox[{"rmat", ".", "xvec"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"dt", " ", "=", " ", 
      RowBox[{"detf", "[", "xvec", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"dfdt", " ", "=", " ", 
      RowBox[{"Sum", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"D", "[", 
          RowBox[{"dt", ",", 
           RowBox[{"xvec", "[", 
            RowBox[{"[", "k", "]"}], "]"}]}], "]"}], "*", 
         RowBox[{"dxv", "[", 
          RowBox[{"[", "k", "]"}], "]"}]}], ",", 
        RowBox[{"{", 
         RowBox[{"k", ",", "nx"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"nm", " ", "=", " ", "nmatf"}], ";", "\[IndentingNewLine]", 
     RowBox[{"dfdtexp", " ", "=", " ", 
      RowBox[{"Nest", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"SparseArray", "[", 
          RowBox[{"grad", "[", 
           RowBox[{"#", ",", "xvec"}], "]"}], "]"}], "&"}], ",", "dfdt", ",", 
        "nm"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"sol", " ", "=", " ", 
      RowBox[{"ToRules", "[", 
       RowBox[{"Reduce", "[", 
        RowBox[{"Thread", "[", 
         RowBox[{
          RowBox[{"Flatten", "[", "dfdtexp", "]"}], "\[Equal]", "0"}], "]"}], 
        "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"matsol", " ", "=", " ", 
      RowBox[{"rmat", " ", "/.", " ", "sol"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"matlist", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"SparseArray", "[", 
         RowBox[{"D", "[", 
          RowBox[{"matsol", ",", "#"}], "]"}], "]"}], "&"}], " ", "/@", " ", 
       "rvars"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"SparseArray", "[", 
      RowBox[{"Select", "[", 
       RowBox[{"matlist", ",", 
        RowBox[{
         RowBox[{
          RowBox[{"Max", "[", 
           RowBox[{"Abs", "[", 
            RowBox[{"Flatten", "[", 
             RowBox[{"Normal", "[", "#", "]"}], "]"}], "]"}], "]"}], ">", 
          "0"}], "&"}]}], "]"}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.6214963238394537`*^9, 3.6214968001817293`*^9}, {
  3.621497514842904*^9, 3.621497518971344*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDHDet", "[", 
   RowBox[{"1", ",", "np_", ",", "x_"}], "]"}], " ", ":=", " ", 
  RowBox[{"First", "[", "x", "]"}]}]], "Input",
 CellChangeTimes->{{3.621502835901156*^9, 3.621502842528039*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDHDet", "[", 
   RowBox[{"2", ",", "np_", ",", "x_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"xs1", ",", "xs2", ",", "xv"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"xs1", " ", "=", " ", 
      RowBox[{"x", "[", 
       RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"xs2", " ", "=", " ", 
      RowBox[{"x", "[", 
       RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"xv", " ", "=", " ", 
      RowBox[{"x", "[", 
       RowBox[{"[", 
        RowBox[{"3", ";;"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"xs1", "*", "xs2"}], " ", "-", " ", 
      RowBox[{"xv", ".", "xv"}]}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.6214974088294973`*^9, 3.621497452796036*^9}, {
  3.621502508990847*^9, 3.621502551107658*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDHDet", "[", 
   RowBox[{"3", ",", "np_", ",", "x_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "n", ",", "mtab", ",", "xs1", ",", "xs2", ",", "xs3", ",", "xv1", ",", 
      "xv2", ",", "xv3"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"n", " ", "=", " ", 
      RowBox[{"2", "^", "np"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"mtab", " ", "=", " ", 
      RowBox[{"CDUnitMultTab", "[", "np", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"xs1", " ", "=", " ", 
      RowBox[{"x", "[", 
       RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"xs2", " ", "=", " ", 
      RowBox[{"x", "[", 
       RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"xs3", " ", "=", " ", 
      RowBox[{"x", "[", 
       RowBox[{"[", "3", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"xv3", " ", "=", " ", 
      RowBox[{"x", "[", 
       RowBox[{"[", 
        RowBox[{"4", ";;", 
         RowBox[{"n", "+", "3"}]}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"xv2", " ", "=", " ", 
      RowBox[{"x", "[", 
       RowBox[{"[", 
        RowBox[{
         RowBox[{"n", "+", "4"}], ";;", 
         RowBox[{
          RowBox[{"2", "n"}], "+", "3"}]}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"xv1", " ", "=", " ", 
      RowBox[{"x", "[", 
       RowBox[{"[", 
        RowBox[{
         RowBox[{
          RowBox[{"2", "n"}], "+", "4"}], ";;", 
         RowBox[{
          RowBox[{"3", "n"}], "+", "3"}]}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"xs1", "*", "xs2", " ", "*", "xs3"}], " ", "-", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"xs1", "*", 
         RowBox[{"(", 
          RowBox[{"xv1", ".", "xv1"}], ")"}]}], "+", 
        RowBox[{"xs2", "*", 
         RowBox[{"(", 
          RowBox[{"xv2", ".", "xv2"}], ")"}]}], "+", 
        RowBox[{"xs3", "*", 
         RowBox[{"(", 
          RowBox[{"xv3", ".", "xv3"}], ")"}]}]}], ")"}], " ", "+", " ", 
      RowBox[{"2", "*", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"MultWithTab", "[", 
          RowBox[{"mtab", ",", "xv1", ",", "xv2"}], "]"}], ".", 
         RowBox[{"CDConjFast", "[", "xv3", "]"}]}], ")"}]}]}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.6214974088294973`*^9, 3.621497452796036*^9}, {
  3.621502508990847*^9, 3.621502777308958*^9}, {3.621503051544262*^9, 
  3.621503051729701*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDHGDAMAlgebra", "[", 
   RowBox[{"m_", ",", "np_"}], "]"}], " ", ":=", " ", 
  RowBox[{"GenDetDiffAMAlgebra", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"CDHDet", "[", 
      RowBox[{"m", ",", "np", ",", "#"}], "]"}], "&"}], ",", 
    RowBox[{"CDHMVals", "[", 
     RowBox[{"m", ",", "np"}], "]"}], ",", "m"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.621497294181546*^9, 3.621497397045395*^9}, {
  3.621497470579245*^9, 3.621497474636435*^9}, {3.6215009489275293`*^9, 
  3.621500950917019*^9}, {3.621502910351036*^9, 3.6215029259668207`*^9}}]
}, Closed]]
}, Closed]]
},
WindowSize->{633, 733},
WindowMargins->{{Automatic, 9}, {5, Automatic}},
ShowSelection->True,
FrontEndVersion->"10.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (September 9, \
2014)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 97, 1, 64, "Section"],
Cell[667, 25, 9907, 233, 3165, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[10611, 263, 94, 1, 64, "Section"],
Cell[10708, 266, 153, 3, 28, "Input"],
Cell[10864, 271, 1456, 36, 148, "Input"],
Cell[12323, 309, 344, 9, 28, "Input"],
Cell[12670, 320, 153, 3, 28, "Input"],
Cell[12826, 325, 2454, 61, 199, "Input"],
Cell[15283, 388, 174, 4, 28, "Input"],
Cell[15460, 394, 395, 9, 28, "Input"],
Cell[15858, 405, 370, 8, 28, "Input"],
Cell[16231, 415, 243, 6, 28, "Input"],
Cell[16477, 423, 378, 9, 28, "Input"],
Cell[16858, 434, 194, 4, 28, "Input"],
Cell[17055, 440, 538, 13, 28, "Input"],
Cell[17596, 455, 283, 7, 28, "Input"],
Cell[17882, 464, 270, 6, 28, "Input"],
Cell[18155, 472, 329, 7, 28, "Input"],
Cell[18487, 481, 368, 9, 28, "Input"],
Cell[18858, 492, 133, 2, 28, "Input"],
Cell[18994, 496, 4555, 108, 352, "Input"],
Cell[23552, 606, 197, 4, 28, "Input"],
Cell[23752, 612, 736, 19, 63, "Input"],
Cell[24491, 633, 3907, 92, 301, "Input"],
Cell[28401, 727, 638, 17, 46, "Input"],
Cell[29042, 746, 576, 16, 46, "Input"],
Cell[29621, 764, 931, 27, 80, "Input"],
Cell[30555, 793, 1426, 35, 148, "Input"],
Cell[31984, 830, 542, 15, 46, "Input"],
Cell[32529, 847, 491, 14, 46, "Input"],
Cell[33023, 863, 345, 8, 28, "Input"],
Cell[33371, 873, 2198, 57, 216, "Input"],
Cell[35572, 932, 250, 7, 28, "Input"],
Cell[35825, 941, 119, 2, 28, "Input"],
Cell[35947, 945, 236, 5, 28, "Input"],
Cell[36186, 952, 2982, 70, 284, "Input"],
Cell[39171, 1024, 262, 6, 28, "Input"],
Cell[39436, 1032, 514, 12, 28, "Input"],
Cell[39953, 1046, 339, 8, 28, "Input"],
Cell[40295, 1056, 510, 12, 28, "Input"],
Cell[40808, 1070, 215, 4, 28, "Input"],
Cell[41026, 1076, 4767, 113, 369, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[45830, 1194, 151, 2, 50, "Section"],
Cell[CellGroupData[{
Cell[46006, 1200, 102, 1, 44, "Subsection"],
Cell[46111, 1203, 209, 4, 28, "Input"],
Cell[46323, 1209, 378, 9, 28, "Input"],
Cell[46704, 1220, 136, 2, 28, "Input"],
Cell[46843, 1224, 435, 12, 28, "Input"],
Cell[47281, 1238, 135, 2, 28, "Input"],
Cell[47419, 1242, 526, 15, 46, "Input"],
Cell[47948, 1259, 350, 9, 46, "Input"],
Cell[48301, 1270, 510, 14, 46, "Input"],
Cell[48814, 1286, 250, 5, 28, "Input"],
Cell[49067, 1293, 516, 14, 46, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[49620, 1312, 112, 1, 36, "Subsection"],
Cell[49735, 1315, 511, 15, 28, "Input"],
Cell[50249, 1332, 259, 5, 28, "Input"],
Cell[50511, 1339, 264, 7, 28, "Input"],
Cell[50778, 1348, 335, 9, 28, "Input"],
Cell[51116, 1359, 270, 7, 28, "Input"],
Cell[51389, 1368, 483, 10, 46, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[51909, 1383, 111, 1, 36, "Subsection"],
Cell[52023, 1386, 235, 6, 28, "Input"],
Cell[52261, 1394, 621, 18, 46, "Input"],
Cell[52885, 1414, 204, 5, 28, "Input"],
Cell[53092, 1421, 315, 8, 28, "Input"],
Cell[53410, 1431, 383, 10, 28, "Input"],
Cell[53796, 1443, 321, 8, 28, "Input"],
Cell[54120, 1453, 197, 4, 28, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[54354, 1462, 108, 1, 36, "Subsection"],
Cell[54465, 1465, 722, 20, 63, "Input"],
Cell[55190, 1487, 699, 19, 63, "Input"],
Cell[55892, 1508, 390, 9, 28, "Input"],
Cell[56285, 1519, 432, 11, 28, "Input"],
Cell[56720, 1532, 369, 9, 28, "Input"],
Cell[57092, 1543, 199, 4, 28, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[57328, 1552, 109, 1, 36, "Subsection"],
Cell[57440, 1555, 551, 16, 46, "Input"],
Cell[57994, 1573, 410, 9, 28, "Input"],
Cell[58407, 1584, 417, 9, 28, "Input"],
Cell[58827, 1595, 471, 12, 63, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[59335, 1612, 107, 1, 36, "Subsection"],
Cell[59445, 1615, 240, 5, 28, "Input"],
Cell[59688, 1622, 336, 10, 28, "Input"],
Cell[60027, 1634, 493, 14, 46, "Input"],
Cell[60523, 1650, 186, 4, 28, "Input"],
Cell[60712, 1656, 226, 4, 28, "Input"],
Cell[60941, 1662, 208, 4, 28, "Input"],
Cell[61152, 1668, 157, 3, 28, "Input"],
Cell[61312, 1673, 521, 13, 63, "Input"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[61882, 1692, 236, 3, 50, "Section"],
Cell[CellGroupData[{
Cell[62143, 1699, 168, 2, 44, "Subsection"],
Cell[62314, 1703, 853, 19, 80, "Input"],
Cell[63170, 1724, 2721, 66, 216, "Input"],
Cell[65894, 1792, 1148, 30, 114, "Input"],
Cell[67045, 1824, 1185, 29, 80, "Input"],
Cell[68233, 1855, 164, 3, 28, "Input"],
Cell[68400, 1860, 176, 4, 28, "Input"],
Cell[68579, 1866, 228, 6, 28, "Input"],
Cell[68810, 1874, 286, 7, 28, "Input"],
Cell[69099, 1883, 136, 2, 28, "Input"],
Cell[69238, 1887, 468, 13, 46, "Input"],
Cell[69709, 1902, 214, 5, 28, "Input"],
Cell[69926, 1909, 1017, 28, 80, "Input"],
Cell[70946, 1939, 442, 8, 28, "Input"],
Cell[71391, 1949, 327, 8, 46, "Input"],
Cell[71721, 1959, 170, 4, 28, "Input"],
Cell[71894, 1965, 189, 4, 28, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[72120, 1974, 163, 2, 36, "Subsection"],
Cell[72286, 1978, 285, 6, 46, "Input"],
Cell[72574, 1986, 2659, 65, 233, "Input"],
Cell[75236, 2053, 1132, 29, 97, "Input"],
Cell[76371, 2084, 250, 5, 28, "Input"],
Cell[76624, 2091, 217, 4, 28, "Input"],
Cell[76844, 2097, 228, 5, 28, "Input"],
Cell[77075, 2104, 2947, 74, 233, "Input"],
Cell[80025, 2180, 682, 15, 63, "Input"],
Cell[80710, 2197, 266, 5, 28, "Input"],
Cell[80979, 2204, 266, 5, 28, "Input"],
Cell[81248, 2211, 166, 3, 28, "Input"],
Cell[81417, 2216, 136, 2, 28, "Input"],
Cell[81556, 2220, 266, 7, 28, "Input"],
Cell[81825, 2229, 164, 3, 28, "Input"],
Cell[81992, 2234, 1453, 37, 148, "Input"],
Cell[83448, 2273, 224, 5, 28, "Input"],
Cell[83675, 2280, 1024, 24, 80, "Input"],
Cell[84702, 2306, 346, 8, 63, "Input"],
Cell[85051, 2316, 2145, 54, 250, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[87233, 2375, 163, 2, 36, "Subsection"],
Cell[87399, 2379, 393, 9, 28, "Input"],
Cell[87795, 2390, 2687, 70, 216, "Input"],
Cell[90485, 2462, 2815, 68, 216, "Input"],
Cell[93303, 2532, 1279, 31, 80, "Input"],
Cell[94585, 2565, 380, 7, 46, "Input"],
Cell[94968, 2574, 174, 4, 28, "Input"],
Cell[95145, 2580, 348, 8, 46, "Input"],
Cell[95496, 2590, 156, 3, 28, "Input"],
Cell[95655, 2595, 1711, 45, 148, "Input"],
Cell[97369, 2642, 2097, 51, 199, "Input"],
Cell[99469, 2695, 772, 18, 63, "Input"],
Cell[100244, 2715, 1003, 25, 80, "Input"],
Cell[101250, 2742, 3007, 77, 267, "Input"],
Cell[104260, 2821, 187, 3, 28, "Input"],
Cell[104450, 2826, 234, 4, 28, "Input"],
Cell[104687, 2832, 3211, 79, 318, "Input"],
Cell[107901, 2913, 210, 5, 28, "Input"],
Cell[108114, 2920, 210, 5, 28, "Input"],
Cell[108327, 2927, 259, 6, 28, "Input"],
Cell[108589, 2935, 261, 6, 28, "Input"],
Cell[108853, 2943, 928, 28, 97, "Input"],
Cell[109784, 2973, 298, 7, 28, "Input"],
Cell[110085, 2982, 299, 7, 28, "Input"],
Cell[110387, 2991, 347, 8, 28, "Input"],
Cell[110737, 3001, 351, 8, 28, "Input"],
Cell[111091, 3011, 287, 7, 46, "Input"],
Cell[111381, 3020, 839, 23, 63, "Input"],
Cell[112223, 3045, 864, 24, 63, "Input"],
Cell[113090, 3071, 503, 10, 63, "Input"],
Cell[113596, 3083, 10499, 255, 862, "Input"],
Cell[124098, 3340, 1173, 34, 97, "Input"],
Cell[125274, 3376, 281, 8, 28, "Input"],
Cell[125558, 3386, 176, 3, 28, "Input"],
Cell[125737, 3391, 278, 8, 28, "Input"],
Cell[126018, 3401, 578, 18, 28, "Input"],
Cell[126599, 3421, 1963, 56, 97, "Input"],
Cell[128565, 3479, 5891, 147, 369, "Input"],
Cell[134459, 3628, 447, 14, 28, "Input"],
Cell[134909, 3644, 1232, 31, 63, "Input"],
Cell[136144, 3677, 3875, 102, 250, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[140056, 3784, 159, 2, 36, "Subsection"],
Cell[140218, 3788, 232, 5, 28, "Input"],
Cell[140453, 3795, 236, 5, 28, "Input"],
Cell[140692, 3802, 796, 22, 80, "Input"],
Cell[141491, 3826, 189, 4, 28, "Input"],
Cell[141683, 3832, 480, 14, 28, "Input"],
Cell[142166, 3848, 246, 6, 28, "Input"],
Cell[142415, 3856, 2260, 56, 233, "Input"],
Cell[144678, 3914, 192, 4, 28, "Input"],
Cell[144873, 3920, 1577, 38, 97, "Input"],
Cell[146453, 3960, 1633, 40, 97, "Input"],
Cell[148089, 4002, 2153, 51, 148, "Input"],
Cell[150245, 4055, 378, 6, 46, "Input"],
Cell[150626, 4063, 874, 26, 63, "Input"],
Cell[151503, 4091, 443, 11, 46, "Input"],
Cell[151949, 4104, 352, 10, 28, "Input"],
Cell[152304, 4116, 3506, 82, 301, "Input"],
Cell[155813, 4200, 3221, 79, 318, "Input"],
Cell[159037, 4281, 228, 5, 28, "Input"],
Cell[159268, 4288, 953, 25, 114, "Input"],
Cell[160224, 4315, 2573, 69, 250, "Input"],
Cell[162800, 4386, 580, 13, 46, "Input"]
}, Closed]]
}, Closed]]
}
]
*)

(* End of internal cache information *)
